#ifndef SDSENSOR_H
#define SDSENSOR_H

#include "spi.h"
#include "gpiopin.h"
#include "taskhandler.h"
#include "lowpower.h"
#include "lsm6dsox_reg.h"
#include "exti.h"

#include <FreeRTOS.h>
#include <semphr.h>

#define SENSOR_ENABLED 1 //!< Value for the TurnSwitchEnable setting.

#define DEFAULT_WAKEUP_THRESHOLD   100   //!< Wakeup threshold in milli-g.
#define DEFAULT_WAKEUP_DURATION    1     //!< Wakeup samples.
#define DEFAULT_ROTATION_THRESHOLD 30    //!< Empty detect threshold in degrees.
#define DEFAULT_ROTATION_TIMEOUT   30000 //!< Empty detect timeout in milliseconds.

class C6DSensor : public CTaskHandler, public CLowPowerTask, public CGPIOInterrupt
{
  public:
    C6DSensor();

    void              Setup(CSpi* spi, SGPIOPin cs, SGPIOPin int2);
    void              Start();

    bool              Enabled() { return m_enabled; }

  private:
    virtual void      GPIOIRQHandler(SGPIOPin pin);
    virtual void      Process();

    bool              Initialize();
    bool              CheckPresent(bool settingenabled);
    bool              CheckDeviceID(bool present);
    void              SetupWakeupOnAcceleration();
    bool              ReadAcceleration(float acceleration[3]);
    bool              ReadWakeup();
    void              EnableGyro();
    void              DisableGyro();
    void              EnableGyroDataReadyInterrupt();
    void              DisableGyroDataReadyInterrupt();
    bool              WaitGyroDataRready();
    bool              CheckEmptied(int64_t threshold, float rotation[3],
                                   uint32_t* previoustime, uint32_t* ignore);

    void              MeasureRotation();

    void              Select();
    void              Deselect();

    static int32_t    SWrite(void *handle, uint8_t reg, const uint8_t *bufp, uint16_t len);
    static int32_t    SRead(void *handle, uint8_t reg, uint8_t *bufp, uint16_t len);

    int32_t           Write(uint8_t reg, const uint8_t *bufp, uint16_t len);
    int32_t           Read(uint8_t reg, uint8_t *bufp, uint16_t len);

    bool              m_enabled;

    CSpi*             m_spi;
    SGPIOPin          m_cs;
    SGPIOPin          m_int2;
    stmdev_ctx_t      m_sensor;
    bool              m_wakeup;
    bool              m_acceldetected;

    SemaphoreHandle_t m_int2signal;
};

#endif //SDSENSOR_H
