#ifndef BARROWSPIFLASHADDRS_H
#define BARROWSPIFLASHADDRS_H

/*! Number of bytes reserved per stored registration in the ringbuffer,
    it should be a factor of the flash sector size, and not be
    larger than one sector of the flash.*/
#define REG_RINGBUFFER_ELEMENT_SIZE    512

//! Number of registrations that can be stored in the ringbuffer.
#define REG_RINGBUFFER_MAX_MESSAGES    (10 * 1024)

/*! Number of bytes total needed for the ringbuffer, this should be
    a multiple of the registration size, and a multiple of the flash sector size.*/
#define REG_RINGBUFFER_TOTAL_SIZE      (REG_RINGBUFFER_ELEMENT_SIZE * REG_RINGBUFFER_MAX_MESSAGES)

/*! Position of the ringbuffer in the SPI flash, starting at the end of the flash.
    The stm32wss library reserves space in the SPI flash starting at the beginning.
    This way, collisions are avoided.
    The start position of the ringbuffer is calculated like this:
    uint32_t position = g_controller.SPIFlash().Size() - REG_RINGBUFFER_TOTAL_SIZE - REG_RINGBUFFER_ADDRESS;
 */
#define REG_RINGBUFFER_ADDRESS         0


/*! Number of bytes reserved per stored status in the ringbuffer,
    it should be a factor of the flash sector size, and not be
    larger than one sector of the flash.*/
#define STATUS_RINGBUFFER_ELEMENT_SIZE 512

//! Number of status messages that can be stored in the ringbuffer.
#define STATUS_RINGBUFFER_MAX_MESSAGES (10 * 1024)

/*! Number of bytes total needed for the ringbuffer, this should be
    a multiple of the status message size, and a multiple of the flash sector size.*/
#define STATUS_RINGBUFFER_TOTAL_SIZE   (STATUS_RINGBUFFER_ELEMENT_SIZE * STATUS_RINGBUFFER_MAX_MESSAGES)

/*! Position of the ringbuffer in the SPI flash, starting at the end of the flash.
    The stm32wss library reserves space in the SPI flash starting at the beginning.
    This way, collisions are avoided.
    The start position of the ringbuffer is calculated like this:
    uint32_t position = g_controller.SPIFlash().Size() - STATUS_RINGBUFFER_TOTAL_SIZE - STATUS_RINGBUFFER_ADDRESS;
 */
#define STATUS_RINGBUFFER_ADDRESS      (REG_RINGBUFFER_ADDRESS + REG_RINGBUFFER_TOTAL_SIZE)


/*! Space reserved for downloading the authorization list.*/
#define AUTHLIST_DOWNLOAD_SIZE         (1024 * 1024)

/*! Address of the authorization list, offset from the end of the SPI flash.*/
#define AUTHLIST_DOWNLOAD_ADDRESS      (STATUS_RINGBUFFER_ADDRESS + STATUS_RINGBUFFER_TOTAL_SIZE)


/*! Space reserved for downloading a file with display images.*/
#define IMAGES_DOWNLOAD_SIZE           (256 * 1024)

/*! Address of the display images file, offset from the end of the SPI flash.*/
#define IMAGES_DOWNLOAD_ADDRESS        (AUTHLIST_DOWNLOAD_ADDRESS + AUTHLIST_DOWNLOAD_SIZE)

#endif //BARROWSPIFLASHADDRS_H
