#ifndef CONTROLLER_H
#define CONTROLLER_H

#include "servercomm.h"
#include "reader.h"
#include "userinterface.h"
#include "spiflash_S25FL512S.h"
#include "settings.h"
#include "deposit.h"
#include "deposittimer.h"
#include "inputs.h"
#include "i2c.h"
#include "registrations.h"
#include "inputmonitor.h"
#include "authlist.h"
#include "lowpower.h"
#include "status.h"
#include "batterycharger.h"
#include "selftest.h"
#include "externalrtc.h"
#include "guileds.h"
#include "adc.h"
#include "lowpowerscheduler.h"
#include "spi.h"
#include "6dsensor.h"
#include "pcbversion.h"

/*! Default setting in minutes that the microcontroller
    can be awake before forcing a reboot.*/
#define DEFAULT_MAX_AWAKE_TIME (3 * 60)

/*! Lowest allowed value in minutes that the microcontroller
    can be awake before forcing a reboot.*/
#define LOWEST_MAX_AWAKE_TIME (1 * 60)

/*! Highest allowed value in minutes that the microcontroller
    can be awake before forcing a reboot.*/
#define HIGHEST_MAX_AWAKE_TIME (7 * 24 * 60)

class CController
{
  public:
    CController();

    void                Setup();
    void                PostStartup();

    CServerComm&        ServerComm()     { return m_servercomm;     }
    CReader&            Reader()         { return m_reader;         }
    CSPIFlash&          SPIFlash()       { return m_spiflash;       }
    CKeyValueStorage&   Settings()       { return m_settings;       }
    CDeposit&           Deposit()        { return m_deposit;        }
    CDepositTimer&      DepositTimer()   { return m_deposittimer;   }
    CUserInterface&     UserInterface()  { return m_userinterface;  }
    CInputs&            Inputs()         { return m_inputs;         }
    CRegistrations&     Registrations()  { return m_registrations;  }
    CStatus&            Status()         { return m_status;         }
    CInputMonitor&      InputMonitor()   { return m_inputmonitor;   }
    CAuthList&          AuthList()       { return m_authlist;       }
    CLowPower&          LowPower()       { return m_lowpower;       }
    CBatteryCharger&    BatteryCharger() { return m_batterycharger; }
    SBatteryInfo&       BatteryInfo()    { return m_batteryinfo;    }
    CExternalRTC&       ExternalRTC()    { return m_externalrtc;    }
    CSelfTest&          SelfTest()       { return m_selftest;       }
    CGUILeds&           GUILeds()        { return m_guileds;        }
    CAdc&               Adc()            { return m_adc;            }
    CLowPowerScheduler& RTCScheduler()   { return m_rtcscheduler;   }
    C6DSensor&          SixDSensor()     { return m_6dsensor;       }
    CPCBVersion&        PCBVersion()     { return m_pcbversion;     }

  private:
    void                SetupSoftwareWatchdog();
    void                CheckExternalRTC();

    CServerComm         m_servercomm;
    CReader             m_reader;
    CUserInterface      m_userinterface;
    CSPIFlash_S25FL512S m_spiflash;
    CKeyValueStorage    m_settings;
    CDeposit            m_deposit;
    CDepositTimer       m_deposittimer;
    CInputs             m_inputs;
    CI2C                m_i2c1;
    CI2C                m_i2c2;
    CRegistrations      m_registrations;
    CStatus             m_status;
    CInputMonitor       m_inputmonitor;
    CAuthList           m_authlist;
    CLowPower           m_lowpower;
    CBatteryCharger     m_batterycharger;
    SBatteryInfo        m_batteryinfo;
    CExternalRTC        m_externalrtc;
    CSelfTest           m_selftest;
    CGUILeds            m_guileds;
    CAdc                m_adc;
    CLowPowerScheduler  m_rtcscheduler;
    CSpi                m_spi5;
    C6DSensor           m_6dsensor;
    CPCBVersion         m_pcbversion;
};

extern CController g_controller;

#endif //CONTROLLER_H
