#ifndef DEPOSIT_H
#define DEPOSIT_H

#include "nfcuid.h"
#include "inputs.h"
#include "registrations.h"
#include "lockdriver.h"
#include "authlist.h"
#include "prepaid.h"
#include "filllevelsensor.h"

#include <ctime>
#include <FreeRTOS.h>
#include <semphr.h>

#define INPUT1_DEFAULT_FILL_INC 1.0f //60 liter deposit opening.
#define INPUT2_DEFAULT_FILL_INC 0.5f //30 liter deposit opening.

/*! Default timeout in milliseconds to wait for a deposit to happen.*/
#define DEPOSIT_TIMEOUT_DEFAULT        6000

/*! Minimum timeout in milliseconds to wait for a deposit to happen.*/
#define DEPOSIT_TIMEOUT_MIN            5000

/*! Maximum timeout in milliseconds to wait for a deposit to happen.*/
#define DEPOSIT_TIMEOUT_MAX            30000

/*! Index of the RTC scheduler alarm used to open and close the lock, based on
    the OpenStart and OpenStop settings.*/
#define LOCK_OPEN_ALARM_INDEX          0

class CDeposit
{
  public:
    CDeposit();

    CLockDriver&      LockDriver() { return m_lockdriver; }

    void              Setup();

    void              CheckLockState();
    
    void              CableTest();

    EDepositResult    Process(NFCUID* uid, bool service, bool* registered,
                              ECardType cardtype, CPrepaid* prepaid = nullptr);
    void              ResetFillLevel(bool anim = true);

    bool              CheckDepositInputFromTimer(bool releasehold, bool releasealways);
    void              PowerDown();

    float             FillLevelIncrease(uint32_t input);

    void              RegisterFillLevelSensor();

  private:
    bool              WantedLockState();
    bool              LockStateSettings(char* openstartstr, char* openstopstr,
                                        tm* openstart, tm* openstop);

    bool              CheckBatteryVoltage();
    bool              DepositAllowed(bool service);
    bool              TimeOpenCloseAllowed(tm now);
    bool              BlockStartStopAllowed(tm now);
    bool              ChargePrepaidDeposit(NFCUID* uid, ECardType cardtype,
                                           CPrepaid* prepaid);

    uint32_t          DebounceTime();
    uint32_t          DepositTimeout();
    bool              CheckFillLevel(bool service);
    void              IncreaseFillLevel(bool service, uint32_t input);
    void              IncreaseUnLockedNotOpened();
    void              DecreaseUnLockedNotOpened();

    void              CheckFillLevelSensorSetting();
    void              PowerUpFillLevelSensor();
    void              PowerDownFillLevelSensor();
    void              ReadFillLevelSensor();

    void              PowerUp();

    uint32_t          DepositInput();
    void              ReadRegSwitchtypeSetting();
    bool              TestDepositInput();
    uint8_t           WaitDepositInput(bool wantedlevel, uint32_t debouncetime,
                                       uint32_t timeout);
    void              RecordDepositInputError();

    void              RegisterDeposit(uint8_t opened, bool service, NFCUID* uid,
                                      ECardType cardtype, CPrepaid* prepaid);

    bool              m_restingstate;
    uint8_t           m_startstate;

    CLockDriver       m_lockdriver;

    CFillLevelSensor  m_filllevelsensor;
    bool              m_readfilllevelsensor;

    SemaphoreHandle_t m_mutex;
};

#endif //DEPOSIT_H
