#ifndef DEPOSITTIMER_H
#define DEPOSITTIMER_H

#include <FreeRTOS.h>
#include <semphr.h>

#include "taskhandler.h"
#include "lowpower.h"
#include "exti.h"

enum EDepositTimerCmd
{
  EDepositTimerCmd_None,
  EDepositTimerCmd_Stop_Opened,
  EDepositTimerCmd_Stop_Closed,
  EDepositTimerCmd_Deposit,
};

class CDepositTimer : public CTaskHandler, public CLowPowerTask, public CRTCInterrupt
{
  public:
    CDepositTimer();

    void              Setup();
    void              Start();

    void              BeforeDeposit(bool opened);
    void              AfterDeposit();

    virtual void      RTCIRQHandler(ERTCIRQType irqtype);

  private:
    virtual void      Process();

    void              Stop(EDepositTimerCmd cmd);
    void              Reset();
    void              SetupTimer();
    void              OnRTCWakeup();
    void              CheckStayedOpen(uint32_t stayopen, bool opened);

    void              SetWakeup();
    uint32_t          RTCDelayTime();

    SemaphoreHandle_t m_signal;  //Semaphore to wake up the CDepositTimer task.
    EDepositTimerCmd  m_cmd;
};

#endif //DEPOSITTIMER_H
