#ifndef DISPLAY_H
#define DISPLAY_H

#include "inclstdint.h"
#include "lvgl.h"
#include "orientation.h"

#define LCD_WIDTH   128
#define LCD_HEIGHT  128
#define LCD_ROWSIZE 8   //!< Number of pixels per row.
#define LCD_ROWS    (LCD_HEIGHT / LCD_ROWSIZE)
#define LCD_BPP     2   //!< Bits per pixel.

class CDisplay
{
  public:
    CDisplay();

    void          Setup(EOrientation orientation);
    lv_obj_t*     Screen() { return m_scr; }

    void          Enable();
    void          Disable();

  private:
    void          SetupLvGL();
    void          SetupFMC();

    void          Initialize();
    void          Clear();
    void          Test();

    static void   Flush(lv_disp_drv_t * disp, const lv_area_t* area, lv_color_t* color);
    static void   Rounder(lv_disp_drv_t* disp, lv_area_t* area);

    EOrientation  m_orientation;

    static void   (*TransformFunction(EOrientation orientation))(uint32_t*, uint32_t*);
    static void   TransformTop(uint32_t* xcoord, uint32_t* ycoord);
    static void   TransformRight(uint32_t* xcoord, uint32_t* ycoord);
    static void   TransformBottom(uint32_t* xcoord, uint32_t* ycoord);
    static void   TransformLeft(uint32_t* xcoord, uint32_t* ycoord);

    lv_disp_buf_t m_dispbuf;
    lv_color_t    m_buf[LCD_WIDTH * LCD_HEIGHT];
    lv_disp_drv_t m_dispdrv;
    lv_obj_t*     m_scr;
};

#endif //DISPLAY_H
