#ifndef INPUTMONITOR_H
#define INPUTMONITOR_H

#include "taskhandler.h"
#include "inputs.h"
#include "lowpower.h"
#include "lowpowerscheduler.h"

/*! Flag in FLAGS_REGISTER that indicates the
    previous state of the fill level switch.*/
#define FILL_LEVEL_SWITCH_FLAG 0x80000000

/*! Flag in FLAGS_REGISTER that indicates the
    previous state of the door switch.*/
#define DOOR_SWITCH_FLAG 0x400000

/*! Flag in FLAGS_REGISTER that indicates that a
    Status_DoorStayedOpen has been writen. */
#define DOOR_STATUS_FLAG 0x200000

/*! Index of the RTC scheduler alarm to check if the door has been opened for
    some time.*/
#define DOOR_ALARM_INDEX 1

class CInputMonitor : public CTaskHandler, public CLowPowerTask,
                      public CRTCSchedulerCallback
{
  public:
    CInputMonitor();

    void         Setup();
    void         Start();
    bool         CheckDoorSwitch();

  private:
    virtual void Process();
    void         CheckLevelReset(bool oldlevelreset, uint8_t readlevels);
    void         CheckDoor(bool olddoorstate, uint8_t readlevels);

    virtual void RTCSchedulerCallback(uint32_t index);
};

#endif //INPUTMONITOR_H
