#ifndef REGISTRATIONS_H
#define REGISTRATIONS_H

#include <FreeRTOS.h>
#include <semphr.h>

#include "flashringbuffer.h"
#include "nfcuid.h"
#include "accellospiflashaddrs.h"
#include "cbor.h"
#include "authlist.h"
#include "prepaid.h"

enum EDepositResult
{
  DepositResult_DontRegister = -1, //!< Don't register this deposit.
  DepositResult_Opened       = 0,  //!< Deposit opening was opened.
  DepositResult_Closed       = 1,  //!< Deposit opening stayed closed.
  DepositResult_Refused      = 2,  //!< Refused to unlock the deposit opening.
};

class CRegistrations
{
  public:
    CRegistrations();

    void              Setup();
    void              SetupRingBuffer();

    void              Register(NFCUID* uid, EDepositResult result, ECardType cardtype, CPrepaid* prepaid = nullptr, int32_t input = -1);

    uint32_t          Read(CborEncoder* registrations, uint8_t* buf, uint32_t bufsize);
    uint32_t          NumStored();
    void              Clear();
    void              Reset();

  private:
    void              IncreaseRegistrationCounter();

    CFlashRingbuffer  m_flashringbuffer;
    uint8_t           m_buf[REG_RINGBUFFER_ELEMENT_SIZE];

    SemaphoreHandle_t m_mutex;
};

#endif //REGISTRATIONS_H
