#ifndef SETTINGS_H
#define SETTINGS_H

#include "keyvaluestorage.h"

extern const SKeyValueInfo g_settingsinfo[];
extern const uint32_t      g_numsettings;

enum EStorageKey
{
  //Settings that are sent by the server.
  Setting_ServerIp = 0,
  Setting_AccelloType,
  Setting_CouplerType,
  Setting_CounterWarning,
  Setting_CounterMax,
  Setting_Operatingmode,
  Setting_GPRSPassword,
  Setting_GPRSUsername,
  Setting_GPRSAPN,
  Setting_SIMPin,
  Setting_BlockStart,
  Setting_BlockStop,
  Setting_Entity,
  Setting_TurnSwitchEnable,
  Setting_TimeOpen,
  Setting_TimeClose,
  Setting_TimeComm,
  Setting_PowerOnNoTagCounter,
  Setting_BreakCounter,
  Setting_InterruptDelay,            //!< Deposit input debounce time in milliseconds.
  Setting_BatteryLow,
  Setting_OpenStart,
  Setting_OpenStop,
  Setting_Servomotor1TimeOpen,
  Setting_Servomotor1TimeClose,
  Setting_FullSensorWarning1,
  Setting_FullSensorMax1,
  Setting_FullSensorReset1,
  Setting_FullSensorWarning2,
  Setting_FullSensorMax2,
  Setting_FullSensorReset2,
  Setting_FullSensorWarning3,
  Setting_FullSensorMax3,
  Setting_FullSensorReset3,
  Setting_US_sensor,
  Setting_Times_Sensor1,
  Setting_Times_Sensor2,
  Setting_Times_Sensor3,
  Setting_ActiveSIM,
  Setting_GPRSPassword_Internal,
  Setting_GPRSUsername_Internal,
  Setting_GPRSAPN_Internal,
  Setting_SIMPin_Internal,
  Setting_DelayStartbutton,
  Setting_DelayLock,
  Setting_Data_anonymous,
  Setting_ChargerPowerSource,
  Setting_LPW_Period,
  Setting_LPW_AmplitudeEnabled,
  Setting_LPW_AmplitudeDelta,
  Setting_LPW_AmplitudeReference,
  Setting_LPW_AmplitudeAutoAvg,
  Setting_LPW_AmplitudeAutoAvgWeight,
  Setting_LPW_AmplitudeAutoAvgInclMeas,
  Setting_LPW_PhaseEnabled,
  Setting_LPW_PhaseDelta,
  Setting_LPW_PhaseReference,
  Setting_LPW_PhaseAutoAvg,
  Setting_LPW_PhaseAutoAvgWeight,
  Setting_LPW_PhaseAutoAvgInclMeas,
  Setting_MaxAwakeTime,
  Setting_CheckStayOpen,
  Setting_RegSwitchNormallyClosed,
  Setting_CounterInput1,
  Setting_CounterInput2,
  Setting_CostInput1,
  Setting_ImagesURL,
  Setting_GPSFixTime,
  Setting_CreditWarningLevel,
  Setting_ReadFillLevelSensor,
  Setting_HoldLockDelay,
  Setting_DisplayOrientation,
  Setting_Servomotor2TimeOpen,
  Setting_Servomotor2TimeClose,
  Setting_HoldLockRelease,
  Setting_HoldLockReleaseAlways,
  Setting_CheckDoorStayOpen,
  Setting_AccelWakeupThreshold,
  Setting_AccelWakeupDuration,
  Setting_RotationThreshold,
  Setting_RotationTimeout,

  //Status variables that will be updated during runtime.
  RunStatus_RegistrationCounter = 1000, //!< Used as a unique number per registration, is never reset.
  RunStatus_FillLevel_Old,              //!< Deprecated.
  RunStatus_PowerOnNoTag,               //!< Low power card detect without reading a card.
  RunStatus_UnlockedNotOpened,          //!< Deposit opening unlocked but not opened.
  RunStatus_ServerContactFailed,        //!< Tried to contact server but failed.
  RunStatus_LastContactAttempt1,        //!< RSSI and date/time of last server contact attempt.
  RunStatus_LastContactAttempt2,        //!< RSSI and date/time of last server contact attempt.
  RunStatus_LastContactAttempt3,        //!< RSSI and date/time of last server contact attempt.
  RunStatus_LastContactIndex,           //!< Which LastContactAttempt has the most recent info.
  RunStatus_FillLevel,                  //!< Number of current deposits (vulgraad).

  //Status variables that are only used in inactive mode.
  InactiveStatus_ServerComm_Tested = 2000, //!< Server communication has succeeded in test mode.
  InactiveStatus_ServerComm_Attempts,      //!< Number of communication attempts in inactive mode.
  InactiveStatus_ServerComm_TxBytes,       //!< Number of bytes sent in inactive mode.
  InactiveStatus_ServerComm_RxBytes,       //!< Number of bytes received in inactive mode.
  InactiveStatus_SelfTest_Passed,          //!< All self tests have succeeded.
  InactiveStatus_CableTest_Passed,         //!< The external cable test has succeeded.
};

#endif //SETTINGS_H
