#ifndef STATUS_H
#define STATUS_H

#include <FreeRTOS.h>
#include <semphr.h>

#include "flashringbuffer.h"
#include "accellospiflashaddrs.h"
#include "statusvalues.h"
#include "cbor.h"

class CStatus
{
  public:
    CStatus();

    void              Setup();
    void              SetupRingBuffer();

    void              Write(EStatus status, const char* format, ...) __attribute__ ((format (printf, 3, 4)));

    uint32_t          Read(CborEncoder* statuses, uint8_t* buf, uint32_t bufsize);
    uint32_t          NumStored();
    void              Clear();
    void              Reset();

    void              RegisterBootReason();

    //cast operator to a SemaphoreHandle_t reference so that CLock(*this); can be used
    operator          SemaphoreHandle_t&() { return m_mutex; }

  private:
    uint32_t          GetSequenceNumber();
    bool              StoreInactive(EStatus status);

    CFlashRingbuffer  m_flashringbuffer;
    uint8_t           m_buf[STATUS_RINGBUFFER_ELEMENT_SIZE];
    char              m_value[MAX_RINGBUFFER_PAYLOAD_SIZE(STATUS_RINGBUFFER_ELEMENT_SIZE)];

    SemaphoreHandle_t m_mutex;
};

#endif //STATUS_H
