#ifndef USERINTERFACE_H
#define USERINTERFACE_H

enum EUIState
{
  UIState_Off,
  UIState_Locked,
  UIState_Unlocked,
  UIState_Denied,
  UIState_Success,
  UIState_SuccessPrepaid,
  UIState_SuccessPrepaidLowCredit,
  UIState_Error,
  UIState_Full,
  UIState_Empty,
  UIState_ServerComm,
  UIState_ResetFillLevelAnim,
  UIState_InsufficientCredit,
  UIState_CardError,
  UIState_Service,
  UIState_String,
  UIState_Inactive,
};

#include "taskhandler.h"
#include "display.h"
#include "prepaid.h"
#include "images.h"
#include "errorcode.h"

#include <FreeRTOS.h>
#include <semphr.h>

class CUserInterface : public CTaskHandler
{
  public:
    CUserInterface();

    void              Setup();
    void              Start();

    void              Set(EUIState state);
    void              SetError(ErrorCode errorcode);
    void              SetBacklight(bool state);
    void              SetString(const char* str);
    void              WaitStateFinish(EUIState state);
    void              SetPrepaidInfo(CPrepaid* prepaid);

    //cast operator to a SemaphoreHandle_t reference so that CLock(*this); can be used
    operator          SemaphoreHandle_t&() { return m_clientmutex; }

    CImages&          Images() { return m_images; }

    EOrientation      Orientation() { return m_orientation; }

  private:
    void              IndicatePowerOn();
    virtual void      Process();
    void              SetState(EUIState state);
    const char*       StateStr(EUIState state);
    EImage            StateImg(EUIState state);
    void              StartResetFillLevelAnimation();
    void              ProcessResetFillLevelAnimation();
    void              ProcessService();
    void              ProcessInactive();
    void              ProcessPrepaid();
    void              ProcessString(const char* str);
    void              ProcessError(ErrorCode errorcode);

    CDisplay          m_display;
    CImages           m_images;
    EOrientation      m_orientation;
    EUIState          m_state;
    EUIState          m_newstate;
    uint32_t          m_resetfilllevelstate;
    int64_t           m_resetfillleveltime;
    int64_t           m_updatetime;
    lv_obj_t*         m_label;
    int32_t           m_oldprepaidvalue;
    int32_t           m_newprepaidvalue;
    const char*       m_str;
    ErrorCode         m_errorcode;

    //to lock access to m_newstate;
    SemaphoreHandle_t m_mutex;

    //to lock access between clients of this class, so that if an animation
    //is running, another task will not interrupt it
    SemaphoreHandle_t m_clientmutex;

    //! To signal the CUserInterface task that m_newstate has changed.
    SemaphoreHandle_t m_signal;
};

#endif //USERINTERFACE_H
