#include "stm32wss.h"
#include "log.h"
#include "controller.h"
#include "backupregisters.h"

/*! Checks if the ENTER_LOW_POWER_MODE_FLAG is set, if so, enters low power mode.*/
void CheckEnterLowPowerMode()
{
  EnableBackupAccess();
 for (volatile uint32_t i=0; i<100000; i++) {
     // nothing EDWIN
 }
  if (ReadBackupFlag(ENTER_LOW_POWER_MODE_FLAG))
  {
    SetBackupFlag(ENTER_LOW_POWER_MODE_FLAG, false);

    //disable interrupts to prevent anything from interrupting this section
    __disable_irq();

    //disable the wakeup pin, clear the wakeup and standby flags, enable the wakeup pin,
    //if the wakeup pin is already high, a wakeup event is immediately generated,
    //in that case, entering standby mode causes an immediate reboot
    PWR_WakeUpPinCmd(DISABLE);
    PWR_ClearFlag(PWR_FLAG_WU);
    PWR_ClearFlag(PWR_FLAG_SB);
    PWR_WakeUpPinCmd(ENABLE);

    //check if an RTC wakeup timer or alarm event has happened, if so, don't
    //enter standby mode, instead reboot, for some reason the hardware doesn't
    //wake up immediately from entering standby if either of these flags are
    //already set, however there still is a very short race condition when
    //doing this check from software
    if (RTC_GetFlagStatus(RTC_FLAG_WUTF)  ||
        RTC_GetFlagStatus(RTC_FLAG_ALRAF) ||
        RTC_GetFlagStatus(RTC_FLAG_ALRBF))
      NVIC_SystemReset();

    //clear all reset reason flags before entering standby mode,
    //otherwise they stay active when coming out of standby mode
    RCC_ClearFlag();

    //enter standby mode, this function does not return
    PWR_EnterSTANDBYMode();

    //fail-safe
    NVIC_SystemReset();
  }
}

/*! Program entry point, SystemInit() is called before this,
    so when this function is called the processor core already runs at 180 MHz.*/
int main()
{
  CheckEnterLowPowerMode();
  SetApplicationLogLevels(LOGLEVELS, sizeof(LOGLEVELS) / sizeof(LOGLEVELS[0]));
  BoardInit();
}

/*! Called before the FreeRTOS scheduler is started, FreeRTOS is not running here.*/
void PreSchedulerStart()
{
    printf("WTF\n");
}

/*! Called from vApplicationDaemonTaskStartupHook() which is called from stm32wss.
    The FreeRTOS kernel is running here.*/
void Startup()
{
    printf("Edwin1\n");
    g_controller.Setup();
}

/*! Called every 500 ms by the system heartbeat.*/
void HeartBeat()
{
   // printf("Edwin2\n");
}
