#include "nfcuid.h"
#include "log.h"
#include "controller.h"

#include <cstring>
#include <cassert>

/*! Prints the UID of the card, or XXXXXXX if Setting_Data_anonymous is enabled.
    \param level The level to pass to PrintLog.
    \param uid The UID of the card to print.
    \param suffix A string printed after the UID, use nullptr to print nothing.*/
void PrintNFCUID(int level, const NFCUID* uid, const char* suffix /*= nullptr*/)
{
  if (!g_controller.Settings().GetBoolean(Setting_Data_anonymous))
    PrintLogHexBuf(level, uid->uid, uid->size, suffix);
  else
    PrintLog(level, "XXXXXXXX%s", suffix != nullptr ? suffix : "");
}

/*! Reverses an NFC uid and writes it to buf.
    \param uid The uid to convert.
    \param buf The buffer to write the uid into.*/
void ReverseNFCUID(const NFCUID* uid, uint8_t buf[NFC_UID_MAX_SIZE])
{
  assert(uid->size <= NFC_UID_MAX_SIZE);

  memset(buf, 0, NFC_UID_MAX_SIZE);

  for (uint32_t i = 0; i < uid->size; i++)
    buf[i] = uid->uid[uid->size - 1 - i];
}
