#include "pcbversion.h"
#include "log.h"
#include "gpiodescriptions.h"
#include "clock.h"

CPCBVersion::CPCBVersion()
{
  m_pcbversion   = PCBVersion_UNKNOWN;
  m_errorprinted = false;
}

/*! Reads the state of the ADC12_14_OPT2 pin. It determines the PCB version.*/
void CPCBVersion::Read()
{
  //PCB Version 1.6 has resistor R41 placed.
  //PCB Version 1.2 has resistor R42 placed.
  if (GPIO_ReadInput(ADC12_14_OPT2))
  {
    m_pcbversion = PCBVersion_1_6;
    PrintLog(LOGPCBVERSION, "PCB version 1.6\n");
  }
  else
  {
    m_pcbversion = PCBVersion_1_2;
    PrintLog(LOGPCBVERSION, "PCB version 1.2\n");
  }
}

EPCBVersion CPCBVersion::Version()
{
  if (m_pcbversion == PCBVersion_UNKNOWN)
  {
    if (!m_errorprinted)
    {
      PrintLog(LOGPCBVERSION, "ERROR: PCB version requested before PCB version is read\n");
      m_errorprinted = true;
    }
  }

  return m_pcbversion;
}

const char* CPCBVersion::VersionStr()
{
  if (m_pcbversion == PCBVersion_1_2)
    return "1.2";
  else if (m_pcbversion == PCBVersion_1_6)
    return "1.6";
  else
    return "ERROR UNKNOWN PCB VERSION";
}
