#include "settings.h"
#include "servercomm.h"
#include "batterycharger.h"
#include "reader.h"
#include "controller.h"
#include "authlist.h"
#include "lockdriver.h"
#include "deposit.h"
#include "6dsensor.h"

/*! Descriptions of settings, and their default values.*/
const SKeyValueInfo g_settingsinfo[] =
{
  //Settings that are sent by the server.
  { Setting_ServerIp,                     "ServerIp",                     "V2",    CborTextStringType, {.s = SERVERCOMM_FALLBACK_URL}              },
  { Setting_AccelloType,                  "AccelloType",                  "V4",    CborIntegerType,    {.i = LockType_ServoMotor}                  },
  { Setting_CouplerType,                  "CouplerType",                  "V5",    CborIntegerType,    {.i = 0}                                    },
  { Setting_CounterWarning,               "CounterWarning",               "V6",    CborIntegerType,    {.i = 0}                                    },
  { Setting_CounterMax,                   "CounterMax",                   "V7",    CborIntegerType,    {.i = 0}                                    },
  { Setting_Operatingmode,                "Operatingmode",                "V8",    CborIntegerType,    {.i = OperatingMode_NoList}                 },
  { Setting_GPRSPassword,                 "GPRSPassword",                 "V9",    CborTextStringType, {.s = ""}                                   },
  { Setting_GPRSUsername,                 "GPRSUsername",                 "V10",   CborTextStringType, {.s = ""}                                   },
  { Setting_GPRSAPN,                      "GPRSAPN",                      "V11",   CborTextStringType, {.s = ""}                                   },
  { Setting_SIMPin,                       "SIMPin",                       "V12",   CborTextStringType, {.s = ""}                                   },
  { Setting_BlockStart,                   "BlockStart",                   "V16",   CborTextStringType, {.s = ""}                                   },
  { Setting_BlockStop,                    "BlockStop",                    "V17",   CborTextStringType, {.s = ""}                                   },
  { Setting_Entity,                       "Entity",                       "V20",   CborIntegerType,    {.i = 0}                                    },
  { Setting_TurnSwitchEnable,             "TurnSwitchEnable",             "V21",   CborIntegerType,    {.i = 0}                                    },
  { Setting_TimeOpen,                     "TimeOpen",                     "V23",   CborTextStringType, {.s = ""}                                   },
  { Setting_TimeClose,                    "TimeClose",                    "V24",   CborTextStringType, {.s = ""}                                   },
  { Setting_TimeComm,                     "TimeComm",                     "V25",   CborTextStringType, {.s = ""}                                   },
  { Setting_PowerOnNoTagCounter,          "PowerOnNoTagCounter",          "V26",   CborIntegerType,    {.i = 10}                                   },
  { Setting_BreakCounter,                 "BreakCounter",                 "V27",   CborIntegerType,    {.i = 5}                                    },
  { Setting_InterruptDelay,               "InterruptDelay",               "V28",   CborIntegerType,    {.i = 300}                                  },
  { Setting_BatteryLow,                   "BatteryLow",                   "V29",   CborIntegerType,    {.i = 3000}                                 },
  { Setting_OpenStart,                    "OpenStart",                    "V30",   CborTextStringType, {.s = ""}                                   },
  { Setting_OpenStop,                     "OpenStop",                     "V31",   CborTextStringType, {.s = ""}                                   },
  { Setting_Servomotor1TimeOpen,          "Servomotor1TimeOpen",          "V34",   CborIntegerType,    {.i = 0}                                    },
  { Setting_Servomotor1TimeClose,         "Servomotor1TimeClose",         "V35",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorWarning1,           "FullSensorWarning1",           "V40",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorMax1,               "FullSensorMax1",               "V41",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorReset1,             "FullSensorReset1",             "V42",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorWarning2,           "FullSensorWarning2",           "V43",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorMax2,               "FullSensorMax2",               "V44",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorReset2,             "FullSensorReset2",             "V45",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorWarning3,           "FullSensorWarning3",           "V46",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorMax3,               "FullSensorMax3",               "V47",   CborIntegerType,    {.i = 0}                                    },
  { Setting_FullSensorReset3,             "FullSensorReset3",             "V48",   CborIntegerType,    {.i = 0}                                    },
  { Setting_US_sensor,                    "US_sensor",                    "V49",   CborIntegerType,    {.i = 0}                                    },
  { Setting_Times_Sensor1,                "Times_Sensor1",                "V50",   CborTextStringType, {.s = "0"}                                  },
  { Setting_Times_Sensor2,                "Times_Sensor2",                "V51",   CborTextStringType, {.s = "0"}                                  },
  { Setting_Times_Sensor3,                "Times_Sensor3",                "V52",   CborTextStringType, {.s = "0"}                                  },
  { Setting_ActiveSIM,                    "ActiveSIM",                    "V53",   CborIntegerType,    {.i = SERVERCOMM_ACTIVE_SIM_DEFAULT}        },
  { Setting_GPRSPassword_Internal,        "GPRSPassword_Internal",        "V54",   CborTextStringType, {.s = ""}                                   },
  { Setting_GPRSUsername_Internal,        "GPRSUsername_Internal",        "V55",   CborTextStringType, {.s = ""}                                   },
  { Setting_GPRSAPN_Internal,             "GPRSAPN_Internal",             "V56",   CborTextStringType, {.s = ""}                                   },
  { Setting_SIMPin_Internal,              "SIMPin_Internal",              "V57",   CborTextStringType, {.s = ""}                                   },
  { Setting_DelayStartbutton,             "DelayStartbutton",             "V58",   CborIntegerType,    {.i = 0}                                    },
  { Setting_DelayLock,                    "DelayLock",                    "V59",   CborIntegerType,    {.i = DEPOSIT_TIMEOUT_DEFAULT}              },
  { Setting_Data_anonymous,               "Data_anonymous",               "V62",   CborBooleanType,    {.b = false}                                },
  { Setting_ChargerPowerSource,           "ChargerPowerSource",           "V63",   CborIntegerType,    {.i = ChargerPowerSource_Solar_PowerSupply} },
  { Setting_LPW_Period,                   "LPW_Period",                   "V64",   CborIntegerType,    {.i = DEFAULT_LPW_PERIOD}                   },
  { Setting_LPW_AmplitudeEnabled,         "LPW_AmplitudeEnabled",         "V65",   CborBooleanType,    {.b = DEFAULT_AMP_ENABLED}                  },
  { Setting_LPW_AmplitudeDelta,           "LPW_AmplitudeDelta",           "V66",   CborIntegerType,    {.i = DEFAULT_AMP_DELTA}                    },
  { Setting_LPW_AmplitudeReference,       "LPW_AmplitudeReference",       "V67",   CborIntegerType,    {.i = DEFAULT_AMP_REFERENCE}                },
  { Setting_LPW_AmplitudeAutoAvg,         "LPW_AmplitudeAutoAvg",         "V68",   CborBooleanType,    {.b = DEFAULT_AMP_AUTOAVG}                  },
  { Setting_LPW_AmplitudeAutoAvgWeight,   "LPW_AmplitudeAutoAvgWeight",   "V69",   CborIntegerType,    {.i = DEFAULT_AMP_AUTOAVGWEIGHT}            },
  { Setting_LPW_AmplitudeAutoAvgInclMeas, "LPW_AmplitudeAutoAvgInclMeas", "V70",   CborBooleanType,    {.b = DEFAULT_AMP_INCLMEAS}                 },
  { Setting_LPW_PhaseEnabled,             "LPW_PhaseEnabled",             "V71",   CborBooleanType,    {.b = DEFAULT_PHASE_ENABLED}                },
  { Setting_LPW_PhaseDelta,               "LPW_PhaseDelta",               "V72",   CborIntegerType,    {.i = DEFAULT_PHASE_DELTA}                  },
  { Setting_LPW_PhaseReference,           "LPW_PhaseReference",           "V73",   CborIntegerType,    {.i = DEFAULT_PHASE_REFERENCE}              },
  { Setting_LPW_PhaseAutoAvg,             "LPW_PhaseAutoAvg",             "V74",   CborBooleanType,    {.b = DEFAULT_PHASE_AUTOAVG}                },
  { Setting_LPW_PhaseAutoAvgWeight,       "LPW_PhaseAutoAvgWeight",       "V75",   CborIntegerType,    {.i = DEFAULT_PHASE_AUTOAVGWEIGHT}          },
  { Setting_LPW_PhaseAutoAvgInclMeas,     "LPW_PhaseAutoAvgInclMeas",     "V76",   CborBooleanType,    {.b = DEFAULT_PHASE_INCLMEAS}               },
  { Setting_MaxAwakeTime,                 "MaxAwakeTime",                 "V77",   CborIntegerType,    {.i = DEFAULT_MAX_AWAKE_TIME}               },
  { Setting_CheckStayOpen,                "CheckStayOpen",                "V78",   CborIntegerType,    {.i = 0}                                    },
  { Setting_RegSwitchNormallyClosed,      "RegSwitchNormallyClosed",      "V79",   CborBooleanType,    {.b = false}                                },
  { Setting_CounterInput1,                "CounterInput1",                "V80",   CborFloatType,      {.f = INPUT1_DEFAULT_FILL_INC}              },
  { Setting_CounterInput2,                "CounterInput2",                "V81",   CborFloatType,      {.f = INPUT2_DEFAULT_FILL_INC}              },
  { Setting_CostInput1,                   "CostInput1",                   "V82",   CborIntegerType,    {.i = 0}                                    },
  { Setting_ImagesURL,                    "ImagesURL",                    "V103",  CborTextStringType, {.s = ""}                                   },
  { Setting_GPSFixTime,                   "GPSFixTime",                   "V113",  CborIntegerType,    {.i = 0}                                    },
  { Setting_CreditWarningLevel,           "CreditWarningLevel",           "V114",  CborIntegerType,    {.i = 0}                                    },
  { Setting_ReadFillLevelSensor,          "ReadFillLevelSensor",          "V93",   CborBooleanType,    {.b = false}                                },
  { Setting_HoldLockDelay,                "HoldLockDelay",                "V83",   CborIntegerType,    {.i = DEFAULT_HOLD_LOCK_DELAY}              },
  { Setting_DisplayOrientation,           "DisplayOrientation",           "V84",   CborIntegerType,    {.i = 0}                                    },
  { Setting_Servomotor2TimeOpen,          "Servomotor2TimeOpen",          "V85",   CborIntegerType,    {.i = 0}                                    },
  { Setting_Servomotor2TimeClose,         "Servomotor2TimeClose",         "V86",   CborIntegerType,    {.i = 0}                                    },
  { Setting_HoldLockRelease,              "HoldLockRelease",              "V87",   CborIntegerType,    {.i = DEFAULT_HOLD_LOCK_RELEASE}            },
  { Setting_HoldLockReleaseAlways,        "HoldLockReleaseAlways",        "V94",   CborBooleanType,    {.b = false}                                },
  { Setting_CheckDoorStayOpen,            "CheckDoorStayOpen",            "V88",   CborIntegerType,    {.i = 0}                                    },
  { Setting_AccelWakeupThreshold,         "AccelWakeupThreshold",         "V89",   CborIntegerType,    {.i = DEFAULT_WAKEUP_THRESHOLD}             },
  { Setting_AccelWakeupDuration,          "AccelWakeupDuration",          "V90",   CborIntegerType,    {.i = DEFAULT_WAKEUP_DURATION}              },
  { Setting_RotationThreshold,            "RotationThreshold",            "V91",   CborIntegerType,    {.i = DEFAULT_ROTATION_THRESHOLD}           },
  { Setting_RotationTimeout,              "RotationTimeout",              "V92",   CborIntegerType,    {.i = DEFAULT_ROTATION_TIMEOUT}             },

  //Status variables that will be updated during runtime.
  { RunStatus_RegistrationCounter,        "RegistrationCounter",          nullptr, CborIntegerType,    {.i = 0}                                    },
  { RunStatus_FillLevel_Old,              "FillLevel_Old",                nullptr, CborIntegerType,    {.i = 0}                                    },
  { RunStatus_PowerOnNoTag,               "PowerOnNoTag",                 nullptr, CborIntegerType,    {.i = 0}                                    },
  { RunStatus_UnlockedNotOpened,          "UnlockedNotOpened",            nullptr, CborIntegerType,    {.i = 0}                                    },
  { RunStatus_ServerContactFailed,        "ServerContactFailed",          nullptr, CborIntegerType,    {.i = 0}                                    },
  { RunStatus_LastContactAttempt1,        "LastContactAttempt1",          nullptr, CborTextStringType, {.s = ""}                                   },
  { RunStatus_LastContactAttempt2,        "LastContactAttempt2",          nullptr, CborTextStringType, {.s = ""}                                   },
  { RunStatus_LastContactAttempt3,        "LastContactAttempt3",          nullptr, CborTextStringType, {.s = ""}                                   },
  { RunStatus_LastContactIndex,           "LastContactIndex",             nullptr, CborIntegerType,    {.i = 0}                                    },
  { RunStatus_FillLevel,                  "FillLevel",                    nullptr, CborFloatType,      {.f = 0.0}                                  },

  //Status variables that are used in inactive mode.
  { InactiveStatus_ServerComm_Tested,     "ServerCommTested",             nullptr, CborBooleanType,    {.b = false}                                },
  { InactiveStatus_ServerComm_Attempts,   "ServerCommAttempts",           nullptr, CborIntegerType,    {.i = 0}                                    },
  { InactiveStatus_ServerComm_TxBytes,    "ServerCommTxBytes",            nullptr, CborIntegerType,    {.i = 0}                                    },
  { InactiveStatus_ServerComm_RxBytes,    "ServerCommRxBytes",            nullptr, CborIntegerType,    {.i = 0}                                    },
  { InactiveStatus_SelfTest_Passed,       "SelfTestPassed",               nullptr, CborBooleanType,    {.b = false}                                },
  { InactiveStatus_CableTest_Passed,      "CableTestPassed",              nullptr, CborBooleanType,    {.b = false}                                },
};

const uint32_t g_numsettings = sizeof(g_settingsinfo) / sizeof(g_settingsinfo[0]);
