\page AuthorisationsList
\brief Access control with the authorisations list.
\tableofcontents

# Accello authorisations list

## Downloading the authorisations list

The Accello downloads an authorisations list when it receives a GetAuthorisationslist command in the reply of a GetCommands command.\n
The URL it downloads it from is sent in the U field of the map that holds the GetAuthorisationslist string.

    {
      "C": "GetAuthorisationslist",
      "U": "http://195.85.182.90:52633/AccelloAPI/Authlist/1000122_20191025_131716.bin"
    }

The authorisations list is stored in the SPI flash during the download. After the download has completed, it will copy it to its internal flash.\n
If the copying to the internal flash does not complete for some reason, the Accello will copy the authorisations list again to the internal flash at the next boot.

## Structure of the authorisations list

The authorisations list is a binary file that stores card UIDs and a type byte per UID.\n
A UID/type pair is stored as 8 bytes little endian, where the 7 least significant bytes form the UID, and the most significant byte is the type byte.

UID | UID | UID | UID | UID | UID | UID | TYPE
----|-----|-----|-----|-----|-----|-----|-----
0   | 1   | 2   | 3   | 4   | 5   | 6   | 7

On a little endian processor, 8 bytes of the authorisation list can directly be read into a uint64\_t,
and the UID and type byte can be extracted using a bitshift and AND mask.\n
The most significant bit of the type byte indicates the size of the UID, if this bit is set, the UID is 7 bytes, if it's cleared, the UID is 4 bytes.

    uint64_t listindex;
    //read 8 bytes from the authorisations list into listindex here

    //extract the type byte
    uint8_t type = listindex >> 56;

    //extract the UID based on the UID size bit
    uint64_t uid;
    if (type & 0x80)
      uid = listindex & 0x00FFFFFFFFFFFFFFULL;
    else
      uid = listindex & 0x00000000FFFFFFFFULL;

    //extract the card type from the type byte
    uint8_t cardtype = type & 0x7F;

## Card type

The 7 card type bits define the function of the card, they are mapped to Stosag types using ::ECardType.
