\page Deposits
\brief The waste deposit process.
\tableofcontents

# Accello deposit process

## Starting a deposit

A deposit starts with the user holding an NFC card to the card reader.
The Accello will read the UID of the card and checks if the UID of the card is present on the whitelist.\n
To do a normal deposit, the UID of the card should be on the \ref AuthorisationsList with the type set to \ref CardType_Registration.

## Executing a deposit

When the type is set to \ref CardType_Registration for the UID of the read card, the Accello will execute a deposit.\n
This is done by calling the \ref CDeposit::Process() method.

It will first check if a deposit is allowed by checking:
  - Current time is within TimeOpen and TimeClose.
  - Current time and date is outside of BlockStart and BlockStop.
  - Current fill level is below CounterMax.

If all those requirements are met, the lock is opened, see \ref Locks. The Accello will then wait for a level change on IN1 for a maximum of 6 seconds.\n

## Detecting a deposit

If a level change is detected on IN1, the user has opened the deposit opening. At that moment the lock is closed immediately.\n
If no level change is detected after 6 seconds, the lock is also closed. IN1 is checked after that in case the deposit opening was opened during the lock close.

If the deposit opening is not opened successfully, an internal counter called UnlockedNotOpened is increased.\n
When UnlockedNotOpened reaches BreakCounter, the server is contacted, because the deposit opening is probably blocked.
UnlockedNotOpened is then reset to 0.

If the deposit opening has been opened successfully, the fill level is increased. UnlockedNotOpened is decreased if higher than 0.\n
If the fill level reaches CounterWarning or CounterMax, the server is contacted.

## Registering a deposit

A CBOR object containing information about the deposit is written to the SPI flash.\n
See \ref Registrations.

## Emptying

When the container is emptied, the fill level needs to be reset to 0, and the server should be contacted.\n
The Accello detects this using input IN3. A high level for at least 1 second on IN3 triggers a fill level reset.\n
The fill level can also be reset by presenting a card who's UID is set to \ref CardType_Emptying on the \ref AuthorisationsList.

When the fill level is reset, an empty symbol is shown on the LCD, the backlight flashes and the speaker beeps.
