\page SecuredCards
\brief Secured cards using MIFARE Ultralight C.
\tableofcontents

# Accello secured cards

## Difference between normal and secured cards.

A normal NFC card is identified by just its UID, and can be copied by anyone.

A secured card is a MIFARE Ultralight C card, with a 3DES authentication key
programmed in by us. This key is unique for each card, it's the first 16 bytes
of a SHA256 HMAC of the UID of the card, using a key hardcoded in the Accello
firmware.

By doing 3DES authentication with the MIFARE Ultralight C card, the Accello can
verify that the card has been produced by us, and the card can verify that it's
communicating with an Accello.
Because the 3DES key can't be read from the card, a secured card can't be copied
by a third party.

## Secured cards on the authorisations list.

Secured cards are added to the authorations list, with an offset of -30 on the
type byte. this means the range of normal cards starts at 101, and the range of
secured cards starts at 71. Other than that they have the same functionality.
An exception to this is the prepaid card, which has type 50, and is always a
secured card.

When an NFC card is presented to the Accello, it will read its UID, and check
if that UID is present on the authorisations list.
If the UID is present as a secured card type , the Accello will perform 3DES
authentication with the card.
If the 3DES authentication succeeds, the card is treated as a normal card, the
function that corresponds to the card type is performed.
If the 3DES authentcation fails, the card is refused.
If the UID is present as a normal unsecured card, the function that corresponds
to the card type is always executed. This means that if secured cards are added
to the authorisations list as normal cards, the security feature is lost.

## Factory-programming secured cards.

Secured cards can be programmed using special Accello firmware. This has to be
done before they are shipped to the customer. They can't be programmed afterwards.

The programming process works in the following way:

- Hold a MIFARE Ultralight Ccard to the Accello, it beeps for 100 ms.
- The Ultralight C card is authenticated with the key, this is in case an
  already programmed card should be re-programmed.
- The card memory is initialized as a prepaid card with all prepaid counters
  set to 0. This makes programming a secured card and prepaid card the same
  process.
- The 3DES key is written to the card.
- The card is locked, so that only after the Accello has performed 3DES
  authentication with the card, the card memory can be written or read.
- The NFC field is turned off and on, 3DES authentication is performed with the
  card, and it's memory is read and verified. This verifies that the card is
  functioning correctly.
- If programming the card has succeeded, the OK symbol is shown the display,
  and the Accello beeps for 100ms.
- If programming the card has failed, an error is shown on the display, and the
  Accello beeps for 10 seconds.
- A registration of type 51 is written, with opened status when a card is
  successfully programmed, and a closed status when programming a card has
  failed. This way the registrations of this Accello can be read to see which
  cards has been programmed with it.

## Flash an Accello with special programming firmware.

### Method 1:
Flash the following firmware into the Accello:
WSS Infocard Systems B.V\DMS - 06 Hardware Ontwikkeling\Producten\Accello\Firmware\program secured or prepaid cards

### Method 2:
In reader.h, set ULTRALIGHT_C_PROGRAMMER_MODE to 1 instead of 0.
Compile the Accello firmware and program it into an Accello.

The Accello will now program any unknown card as a secured/prepaid card.
