\page Servercomm
\tableofcontents
\brief Communication between the Accello and the server.

# How the Accello communicates with a server

## Protocol between Accello and server

The Accello can communicate with a server using HTTP POST.\n
Over HTTP, it sends and received messages encoded as [CBOR](https://cbor.io/).

Because the Accello is a battery powered device, it will only contact the server periodically, or when an important event happens.\n
When it contacts the server, the first thing it does is send a GetCommands message to the server:

    {
      "I": "1000122",      // Device ID.
      "C": "GetCommands",  // Command for the server.
      "N": 36,             // Message sequence number.
      "V": "1.1"           // Current firmware version.
    }

The server will reply with a message that tells the Accello what to do:

    {
      "I": "1000122",                      //Device ID.
      "S": 0,                              //Status, 0 = OK, 9 = ERROR.
      "D": "2019-10-25T11:17:16.6799282Z", //Current date and time in ISO8601 format, used to set the Accello's RTC.
      "C":                                 //Array of commands that should be executed.
        [
          //Send current settings to the server.
          {"C": "PostSettings", "U": null},

          //Get new settings from the server.
          {"C": "GetSettings", "U": null},

          //Send current status to the server.
          {"C": "PostStatus", "U": null},

          //Send registrations to the server.
          {"C": "PostRegistrations", "U": null},

          //Download a new authorisations list.
          {"C": "GetAuthorisationslist", "U": "http://195.85.182.90:52633/AccelloAPI/Authlist/1000122_20191025_131716.bin"}

          //Update firmware.
          {"C": "GetFirmware", "U": "http://rsoffice.net/firmware/Accello/Barrow_1.0.bin"}
        ]
    }

This way, when the Accello contacts the server, the server decides what the Accello does next.\n
The reason for this command structure is that if the Accello firmware has a bug in any of the commands, the command can be disabled from the server.\n
The bug can then be fixed using a firmware update, assuming the GetFirmware command works correctly.\n
A sequence number is attached to each message that the Accello sends to the server, this is mainly used for debugging purposes.

## Commands

### PostSettings
The PostSettings command sends all settings for this Accello to the server.\n
See \ref Settings.

Request:

    {
      "I": "1000122",                      // Device ID.
      "C": "GetCommands",                  // Command for the server.
      "N": 36,                             // Message sequence number.

      //all settings as key-value pairs
    }

Reply:

    {
      "I": "1000123",                      //Device ID.
      "D": "2019-10-25T14:01:31.7147637Z", //Current date and time in ISO8601 format.
      "S": 0,                              //Status, 0 = OK, 9 = ERROR.
      "R": 2                               //TODO: figure out what this means.
    }

### GetSettings
The GetSettings command downloads all settings for this Accello from the server.\n
See \ref Settings.

Request:

    {
      "I": "1000123",                      //Device ID.
      "C": "GetSettings",                  //Command.
      "N": 846                             //Message sequence number.
    }

Reply:

    {
      "I": "1000123",                      //Device ID.
      "S": 0,                              //Status, 0 = OK, 9 = ERROR.

      //all settings as key-value pairs
    }

### PostStatus
The PostStatus command sends the current status of this Accello to the server.\n
See \ref Status.

    TODO: implement and document the status fields

### PostRegistrations
The PostRegistrations command sends the current waste deposit registrations stored in the SPI flash to the server.\n
See \ref Registrations.

Request:

    {
      "I": "1000123",                      //Device ID
      "C": "PostRegistrations",            //Command
      "N": 852,                            //Message sequence number.
      "R":                                 //Registrations array.
        [_ //T: UID.
           //S: 0 = Deposit opening was opened.
           //S: 1 = Deposit opening stayed closed.
           //S: 2 = Accello refused to unlock the deposit opening.
           //D: Date and time in ISO8661 format of the deposit.
           //N: Sequence number of the deposit, used to prove that none were missed.
          {"T": "3A81EECE", "S": 0, "D": "2019-10-25T14:32:08Z", "N": 291},
          {"T": "3A81EECE", "S": 0, "D": "2019-10-25T14:32:16Z", "N": 292},
          {"T": "EAA53A29", "S": 1, "D": "2019-10-25T14:32:24Z", "N": 293}
        ]
    }

Reply:

    {
      "D": "2019-10-25T14:32:44.065676Z",  //Current date and time in ISO8601 format.
      "S": 0,                              //Status, 0 = OK, 9 = ERROR.
      "R": 3                               //Number of registrations accepted by the server.
    }

### GetAuthorisationslist
The GetAuthorisationslist command downloads a new authorisations list using HTTP GET and stores it in the Accello's flash.\n
It downloads the file passed in the U parameter that the server sends with the GetAuthorisationslist command.\n
See \ref AuthorisationsList.

### GetFirmware
The GetFirmware command downloads new firmware for the Accello using HTTP GET, and starts a firmware update.\n
It downloads the file passed in the U parameter that the server sends with the GetFirmware command.\n
After the download has completed, the Accello installs the new firmware.\n
Because installing new firmware requires a reboot, any commands in the command array after the GetFirmware command are not executed.\n
It's recommended to make the GetFirmware command always the last command in the commands array.

### ProductionTest
The ProductionTest message is sent by the Accello itself, when a selftest is done when the device is inactive.
The Accello uses this to confirm that its internet connection works.
If it has received a reply from the server, the Accello will never send this message again.

The message looks like this:
  {
      "I": "1000123",                      //Device ID
      "C": "ProductionTest",               //Command
      "N": 852,                            //Message sequence number.
      "V": 3.8,                            //Firmware version.
      "M": "868446035405956",              //IMEI.
      "D": "89430301901510380494",         //ICCID.
      "R": 22,                             //RSSI.
  }
