\page Settings
\tableofcontents
\brief All settings in the Accello.

# Accello settings

---

## ServerIp
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V2           | TextString | "https://rsoffice.net/accelloAPI/req"       |

URL used to Communicate with the server.
CBOR Commands are sent to this URL using HTTP POST,
and CBOR replies are received from the server.\n
See \ref Servercomm.

TODO: rename to ServerURL

---

## AccelloType
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V4           | Integer    | 20                                          |

Solenoid        = 16
Solenoid_Toggle = 17
Motor           = 18
Bistable_2      = 19
ServoMotor      = 20
Bistable_1      = 21
Press_Engels    = 22
Solenoid_Double = 23
30_60_Liter     = 24
HoldLock        = 25

---

## CouplerType
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V5           | Integer    | 0                                           |

Mifare UID, check op authorisatielijst.\n
Mifare Ntag, check block 5, controle hash, special entity scheepvaart.\n
Mifare WSS, check block 5, controle hash, entity.\n
TODO: figure out which number means what.

---

## CounterWarning
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V6           | Integer    | 0                                           |

When this fill level is reached after a deposit, contact the server that the
container should be emptied.\n
If this setting is 0, the server will not be contacted.

---

## CounterMax
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V7           | Integer    | 0                                           |

When the fill level is at this level or higher, deposits are refused.
A service opening is still allowed.
When the fill level reaches this level after a deposit, the server is contacted.\n
If this setting is 0, deposits are always allowed, and the server is
never contacted.

---

## Operatingmode
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V8           | Integer    | 0                                           |

 * 0 = No list, all cards are accepted as standard deposit if not present on the \ref AuthorisationsList.
 * 1 = Blacklist, not supported, all cards are accepted as standard deposit if not present on the \ref AuthorisationsList.
 * 2 = Whitelist, all cards are checked against the \ref AuthorisationsList, and are refused if not present.

---

## GPRSPassword
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V9           | TextString | ""                                          |

Password to use for the PDP context of the external SIM.

TODO: rename to APNPassword_External

---

## GPRSUsername
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V10          | TextString | ""                                          |

Username to use for the PDP context of the external SIM.

TODO: rename to APNUsername_External

---

## GPRSAPN
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V11          | TextString | ""                                          |

APN to use for the PDP context of the external SIM. 

TODO: rename to APN_External

---

## SIMPin
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V12          | TextString | ""                                          |

PIN to use with the external SIM.

TODO: rename to SIMPin_External

---

## BlockStart
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V16          | TextString | ""                                          |

Timestamp in ISO8601 format at which to start refusing deposits.
Used together with BlockStop to create a period in which nobody is allowed to deposit.

---

## BlockStop
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V17          | TextString | ""                                          |

Timestamp in ISO8601 format at which to end refusing deposits.
Used together with BlockStart to create a period in which nobody is allowed to deposit.

---

## Entity
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V20          | Integer    | 0                                           |

Salt WSS entity.\n
TODO: make a better description.

---

## TurnSwitchEnable
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V21          | Integer    | 0                                           |

Sets the type of level reset switch:

* 0 = Off.
* 1 = Tilt sensor, never implemented.
* 2 = External input 3.
* 3 = Tilt sensor and external input 3, combined with logical AND.
* 4 = Ultrasonic input.

TODO: rename to FillLevelResetInput

---

## TimeOpen
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V23          | TextString | ""                                          |

Start time of day in which deposits are allowed.
Creates a daily window together with TimeClose in which deposits can be done.\n
Format is HH:MM\n
A deposit is always allowed when set to the same time as TimeClose.

---

## TimeClose
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V24          | TextString | ""                                          |

End time of day in which deposits are allowed.
Creates a daily window together with TimeOpen in which deposits can be done.\n
Format is HH:MM\n
A deposit is always allowed when set to the same time as TimeOpen.

---

## TimeComm
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V25          | TextString | ""                                          |

Timestamp in ISO8601 format of when the Accello should contact the server.
This timestamp is updated each time the Accello sends a GetSettings command to the server.
Before going into low power mode, the Accello will check this setting against
the time of its own RTC. If the TimeComm timestamp is in the past, or more than 36 hours
in the future, it will set TimeComm to its current RTC time plus 24 hours.
It will then set its RTC alarm to wake up at this timestamp, after which it will contact
the server again.

---

## PowerOnNoTagCounter
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V26          | Integer    | 10                                          |
 
When the power on no tag counter reaches this value, the server should be contacted.
A high value of the power on no tag counter indicates that there is something wrong
with the low power card detect from the NFC reader.

---

## BreakCounter
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V27          | Integer    | 5                                           |

If the deposit opening has been unlocked but not opened this amount of times,
contact the server, because the deposit opening is probably jammed.

TODO: rename to NotOpenedCounter

---

## InterruptDelay
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Incorrectly | V28          | Integer    | 300                                         |

Debounce value in milliseconds of the registration input, or of the reset input.\n
TODO: figure out what this setting is supposed to do.

---

## BatteryLow
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V29          | Integer    | 3000                                        |

Value in millivolt below which the lock is no longer operated, instead a warning
is shown in the display.\n
TODO: figure out if a service opening should still be allowed.

---

## OpenStart
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V30          | TextString | ""                                          |

Start timestamp in ISO8601 format at which the lock is always opened.
Used together with OpenStop to create a period where everyone is allowed
to deposit without a card.\n
If this time is in the future, the Accello will wake up from standby at this time,
and it will automatically open the lock.

---

## OpenStop
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V31          | TextString | ""                                          |

End timestamp in ISO8601 format at which the lock is always opened.
Used together with OpenStart to create a period where everyone is allowed
to deposit without a card.\n
If this time is in the future, the Accello will wake up from standby at this time,
and it will automatically close the lock.

---

## ServomotorTimeOpen
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V34          | Integer    | 0                                           |

Length of the pulse in microseconds that is sent to the servo to open the lock.

---

## ServomotorTimeClose
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V35          | Integer    | 0                                           |

Length of the pulse in microseconds that is sent to the servo to close the lock.

---

## FullSensorWarning1
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V40          | Integer    | 0                                           |

Currently unused.

---

## FullSensorMax1
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V41          | Integer    | 0                                           |

Currently unused.

---

## FullSensorReset1
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V42          | Integer    | 0                                           |

Currently unused.

---

## FullSensorWarning2
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V43          | Integer    | 0                                           |

Currently unused.

---

## FullSensorMax2
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V44          | Integer    | 0                                           |

Currently unused.

---

## FullSensorReset2
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V45          | Integer    | 0                                           |

Currently unused.

---

## FullSensorWarning3
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V46          | Integer    | 0                                           |

Currently unused.

---

## FullSensorMax3
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V47          | Integer    | 0                                           |

Currently unused.

---

## FullSensorReset3
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V48          | Integer    | 0                                           |

Currently unused.

---

## US_sensor
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V49          | Integer    | 0                                           |

 * 0 = No sensor.
 * 1 = Sensor 1 present.
 * 2 = Sensor 1 and 2 present.
 * 3 = Sensor 1, 2 and 3 present.

---

## Times_Sensor1
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V50          | TextString | 0                                           |

Currently unused.

---

## Times_Sensor2
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V51          | TextString | 0                                           |

Currently unused.

---

## Times_Sensor3
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V52          | TextString | 0                                           |

Currently unused.

---

## ActiveSIM
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V53          | Integer    | Internal (0)                                |

Sets the SIM to use, can be set to the internal soldered SIM,
or to the external SIM in the SIM socket.\n

* 0 = Internal SIM.
* 1 = External SIM.

---

## GPRSPassword_Internal
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V54          | TextString | ""                                          |

Password to use for the PDP context of the internal SIM.

TODO: rename to APNPassword_Internal

---

## GPRSUsername_Internal
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V55          | TextString | ""                                          |

Username to use for the PDP context of the internal SIM.

TODO: rename to APNUsername_Internal

---

## GPRSAPN_Internal
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V56          | TextString | ""                                          |

APN to use for the PDP context of the external SIM. 

TODO: rename to APN_Internal

---

## SIMPin_Internal
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V57          | TextString | ""                                          |

PIN to use with the external SIM.

---

## DelayStartbutton
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| No          | V58          | Integer    | 0                                           |

Will not be implemented, the Accello does not have a start button.

---

## DelayLock
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V59          | Integer    | 6000                                        |

Timeout in milliseconds waiting for the registration switch after the lock has been opened.
When this timeout expires, close the lock, write a registration that no deposit has been detected, and power down.\n
When this setting is 0, a timeout of 6 seconds is used.

---

## Data_anonymous
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V62          | Boolean    | false                                       |

When true, the UID of the card should not be registered anywhere when doing
a regular deposit. The UID of other types of cards should still be registered. See \ref ECardType.

---

## ChargerPowerSource
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V63          | Integer    | 0                                           |

Sets the power source for the internal LiFePo charger.\n

* 0: Power supply or solar panel, the charger defaults are used.\n
     The max_cv_time register is set to 3600 seconds.\n
     The v_recharge_lifepo4 register is set to 3.352427 volt.\n
     The input_undervoltage_setting is set to 4.5 volt.\n
     MPPT is enabled.\n
     The max_charge_time register is set to 0 (off).
\n
* 1: Battery pack, the charger settings are adjusted to use the external
     battery pack in the most efficient way.\n
     The max_cv_time register is set to 1 second.\n
     The v_recharge_lifepo4 register is set to 3.2 volt.\n
     The input_undervoltage_setting is set to 4.5 volt.\n
     MPPT is enabled.\n
     The max_charge_time register is set to 0 (off).
\n
* 2: Solar panel with 12 volt battery, the charger settings are adjusted to
     charge the internal LiFePO battery without over discharging the external
     12 volt battery.\n
     The max_cv_time register is set to 1 second.\n
     The v_recharge_lifepo4 register is set to 3.29 volt.\n
     The input_undervoltage_setting is set to 11.672 volt.\n
     MPPT is disabled.\n
     The max_charge_time register is set to 720 minutes (12 hours).

When the charger charges the battery, it will charge it at 1A up to 3.6 volts.\n
When 3.6 volts is reached, the max_cv_time register (max constant voltage time)
controls how long it will keep the battery at 3.6 volts.\n
Because constant voltage charging is the most inefficient part of the LiFePO charge cycle,
it's set to terminate after 1 second when charging from a battery pack.\n
The max_cv_time register should not be set to 0, because the charger will then keep the battery at 3.6 volts forever.\n


When an input voltage from a power supply, battery or solar panel is connected to the charger,
the charger will always start charging the battery.\n
After the charge has completed and has been terminated by the charger, the v_recharge_lifepo4 register
controls the voltage threshold below which the charger will start charging the battery again,
but only if the input voltage never drops below 4.4 volts.\n


Because recharging the battery often up to 3.6 volts is inefficient, the recharge threshold is set to 3.2 volts
when charging from an external battery pack.\n
This will cause the recharge to happen when the battery has about 20% charge left.\n
This limits the number of times that the charger charges the battery up to 3.6 volts.


When using an external 12 volt battery, the recharge threshold is set to 3.29 volts,
so that the internal LiFePO battery is kept at a reasonable charge level,
without charging it up to 3.6 volts too often.\n
The input_undervoltage_setting is set to 11.627 volts to prevent over
discharging the external 12 volt battery.\n
The charger is allowed to charge for a maximum of 12 hours, after which the charging will stop.\n
Charging will only be re-enabled by firmware if the external 12 volt battery is higher than 12 volts,
or by disconnecting the external battery, waiting a minute for the capacitors to drain, and reconnecting it.

---

## LPW_Period

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V64          | Integer    | 800                                         |

Interval in milliseconds that the NFC reader detects if a new card is present when it's in low power card detect mode.\n
Possible values:\n
* 10
* 20
* 30
* 40
* 50
* 60
* 70
* 80
* 100
* 200
* 300
* 400
* 500
* 600
* 700
* 800

Setting this to a lower value increases idle current consumption:\n

* 800ms: 120µA
* 400ms: 150µA
* 200ms: 180µA
* 100ms: 240µA
* 50ms: 400µA
* 10ms: 1200µA


---

## LPW_AmplitudeEnabled

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V65          | boolean    | false                                       |

Enables amplitude measurement for the NFC reader when it's in low power card detect mode.

---

## LPW_AmplitudeDelta

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V66          | integer    | 6                                           |

Difference in amplitude measurements higher than this causes the NFC reader to generate
a wakeup interrupt when it's in low power card detect mode.\n
Lower values makes the reader more sensitive to changes near the NFC coil, but it also
makes it more prone to wake up without a card present.

---

## LPW_AmplitudeReference

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V67          | integer    | 255                                         |

Reference value for when the reader is in low power card detect mode. If the measured value
is differs more than LPW_AmplitudeDelta from this value, a wakeup interrupt is signaled.
Use 255 to auto detect the reference value.

---

## LPW_AmplitudeAutoAvg

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V68          | boolean    | true                                        |

When true, keeps tracking the reference value based on amplitude measurements.\n
Small changes in amplude will only change the reference.\n
A large change in amplitude will trigger a wakeup.

---

## LPW_AmplitudeAutoAvgWeight

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V69          | integer    | 32                                          |

The weight of the old reference value when updating the reference value with a measurement.\n
A higher value means that the reference value is updated slower.\n

Possible values:
* 4
* 8
* 16
* 32

---

## LPW_AmplitudeAutoAvgInclMeas

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V70          | boolean    | false                                       |

When true, after a wakeup has been triggered, update the reference value with the signal that triggered the wakeup.

---

## LPW_PhaseEnabled

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V71          | boolean    | true                                        |

Enables phase measurement for the NFC reader when it's in low power card detect mode.

---

## LPW_PhaseDelta

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V72          | integer    | 4                                           |

Difference in phase measurements higher than this causes the NFC reader to generate
a wakeup interrupt when it's in low power card detect mode.\n
Lower values makes the reader more sensitive to changes near the NFC coil, but it also
makes it more prone to wake up without a card present.

---

## LPW_PhaseReference

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V73          | integer    | 255                                         |

Reference value for when the reader is in low power card detect mode. If the measured value
is differs more than LPW_PhaseDelta from this value, a wakeup interrupt is signaled.
Use 255 to auto detect the reference value.

---

## LPW_PhaseAutoAvg

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V74          | boolean    | true                                        |

When true, keeps tracking the reference value based on phase measurements.\n
Small changes in amplude will only change the reference.\n
A large change in phase will trigger a wakeup.

---

## LPW_PhaseAutoAvgWeight

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V75          | integer    | 32                                          |

The weight of the old reference value when updating the reference value with a measurement.\n
A higher value means that the reference value is updated slower.\n

Possible values:
* 4
* 8
* 16
* 32

---

## LPW_PhaseAutoAvgInclMeas

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V76          | boolean    | false                                       |

When true, after a wakeup has been triggered, update the reference value with the signal that triggered the wakeup.

---

## MaxAwakeTime

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V77          | integer    | 180                                         |

Maximum time in minutes that the microcontroller is allowed to stay out of low power mode.
The value for this is limited in firmware from 60 minutes to 7 days.\n
After this time expires the system is rebooted using the NVIC_SystemReset() function.\n
This is used as a software watchdog timer, if the system is awake for this long without
going into low power mode, one of the FreeRTOS tasks is probably frozen.\n
A reboot might fix this.

---

## CheckStayOpen

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V78          | integer    | 0                                           |

Sets the time in seconds after a deposit that the Accello will check if the deposit opening is still opened.\n
If so, it will write a \ref Status_DepositOpeningStayedOpen with a value of 1.\n
It will periodically check the deposit opening with an expotentially increasing check time,
the maximum time between checks is 6 hours.\n
If the deposit opening is closed again, the Accello will write a \ref Status_DepositOpeningStayedOpen with a value of 0.

When this setting it set to 0, the check is disabled.

---

## RegSwitchNormallyClosed

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V79          | boolean    | false                                       |

Sets the type of the deposit registration switch.\n
When this setting is false, the registration switch is a normally open type.\n
When this setting is true, the registration switch is a normally closed type.

---

## CounterInput1

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V80          | float      | 1.0                                         |

The value that is added to the internal fill level counter when a deposit is detected on input1.\n
The default is 1.0 for a 60 liter input.

---

## CounterInput2

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V81          | float      | 0.5                                         |

The value that is added to the internal fill level counter when a deposit is detected on input2.\n
The default is 0.5 for a 30 liter input.

---

## CostInput1

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V82          | integer    | 0                                           |

The cost of a prepaid deposit.

---

## ImagesURL

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V103         | TextString | ""                                          |

Sets the URL to download custom display images from.\n
The file is downloaded each time the URL is changed.\n
Set the URL to an empty string to use the default display images.

---

## GPSFixTime

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V113         | integer    | 0                                           |

Time in seconds waiting for a GPS location fix during \ref Servercomm. \n
The GPS location will then be recorded as a GPS Location \ref Status. \n
Recording the GPS location is disabled when this setting is 0.

---

## CreditWarningLevel

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V114         | integer    | 0                                           |

If the credit of a prepaid card is below this value multiplied by CostInput1 after the
deposit has been processed, a low credit warning is shown on the display, and the server
is contacted in case the user has already bought extra credit.

---

## ReadFillLevelSensor

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V93          | Boolean    | false                                       |

When true, the fill level sensor is read when the lock is opened, and during server
communication.\n
The fill level is recorded as a Fill level ultrasonic sensor 1 \ref Status.

---

## HoldLockDelay

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V83          | Integer    | 15000                                       |

When the lock type is set to hold lock, wait this number of milliseconds before powering
the hold magnet. Twice after that this time is doubled and the magnet is powered again.\n
The magnet is powered for a total of 3 times.\n
The magnet is not powered if the deposit input is closed.

---

## DisplayOrientation

| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V84          | Integer    | 0                                           |

Sets the location of the display in relation to the NFC reader coil.\n
Possible values:

- 0, Top
- 1, Right
- 2, Bottom
- 3, Left

---

## Servomotor2TimeOpen
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V85          | Integer    | 0                                           |

Length of the pulse in microseconds that is sent to the second servo.\n
In case of a 30/60 liter lock, this opens and closes the second servo lock.\n
\n
In case of a hold lock, this actuates the hold servo.\n
The open time controls the released position, the close time controls the hold position.

---

## Servomotor2TimeClose
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V86          | Integer    | 0                                           |

Length of the pulse in microseconds that is sent to the second servo.

---

## HoldLockRelease
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V87          | Integer    | 1000                                        |

When the lock type is set to hold lock, this is the time in milliseconds that the
hold magnet is powered.

---

## CheckDoorStayOpen
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V88          | Integer    | 0                                           |

A door switch can be connected to input IN4, to create a high level when the door is
opened. After a high level is detected, an countdown alarm in milliseconds is set to the value
of CheckDoorStayOpen. After this alarm expires, input IN4 is checked again. If the input
is still high, a \ref Status_DoorStayedOpen is written with a value of "1", and the server
is contacted.
If afterwards, a low level appears on IN4, a status \ref Status_DoorStayedOpen with a
value of "0" is written, and the server is contacted.

If this setting is 0, checking the door switch on IN4 is disabled.

---

## HoldLockReleaseAlways
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V94          | boolean    | false                                       |

When the lock type is set to hold lock, always power the hold lock magnet even if the
deposit opening is closed.

---

## AccelWakeupThreshold
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V89          | Integer    | 100                                         |

Wakeup threshold in milli-g of the 6D sensor accelerometer.
Value can be set from 8 to 2000. A value that is too low will cause a lot of wakeups
and will drain the battery.

---

## AccelWakeupDuration
| Implemented | Server field | Type       | Default                                     |
|-------------|--------------|------------|---------------------------------------------|
| Yes         | V90          | Integer    | 1                                           |

Wakeup duration in samples of the 6D sensor accelerometer.
When this number of consecutive samples are above the value set with the
AccelWakeupThreshold setting, the 6D sensor generates an interrupt.
