\page Status
\brief All status fields in the Accello.
\tableofcontents

# Accello status

## Status records

### External battery voltage without load.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 0           | Voltage (millivolts).                         |

This is only recorded when ChargerPowerSource is set to 1 for battery pack.\n
Otherwise, a solar voltage status is recorded.

---

### External battery voltage under load.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 1           | Voltage (millivolts).                         |

---

### Fill level.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 2           | Number of deposits.                           |

---

### Software version.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 3           | Return value of FirmwareVersion().            |

---

### Fill level ultrasonic sensor 1.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 4           | Distance from fill level sensor in meters.    |

---

### PWR rail without load.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 5           | Voltage (millivolts)                          |

Voltage on POWER_RAIL is measured after the boost converter is switched on.

---

### PWR rail under load.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 6           | Voltage (millivolts)                          |

---

### Emptied.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 8           | Fill level when emptied.                      |

The server should be contacted right after this status has been stored.

---

### Number of contact attempts.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 9           | Number of contact attempts                    |

The number of contact attempts it took before the server was reached successfully.\n
If this status is 1, then there were no failed contact attempts.

---

### Battery state.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 11          | Battery state 1 to 4.                         |

Probably reflects the voltage of an external 12V battery.

---

### Soft reset

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 12          | Reset reason.                                 |

Reason why the microcontroller has booted.

Possible values:
* software watchdog timer
* software request
* independent watchdog timer
* windowed watchdog timer
* low-power management
* power-on
* brown-out
* reset pin

There are usually multiple reasons for a reset, but only one status is written per boot.\n
All reset reasons are concatenated separated by a comma and a space.

---

### Power on no tag.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 52          | Number of power on no tag events.             |

Power on no tag happens when the Accello wakes up from standby because of low power card
detect, but it did not read a card. This indicates that there is something wrong with
the low power card detect. Battery life is decreased if this happens often.

---

### Modem RSSI.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 53          | RSSI returned by the modem.                   |

---

### External SIM ICCID.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 54          | SIM ICCID.                                    |

The external SIM ICCID is read and stored as a status every time a connection
to the internet has been made using the modem and the external socketed SIM.

---

### Dial reason.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 55          | Reason that the server was contacted.         |

Reason that the server was contacted.\n
Possible values:\n

* outbound call card
* warning fill level reached
* max fill level reached
* deposit opening blocked
* emptied
* RTC wakeup timer
* RTC alarm

---

### Opened.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 56          | UID                                           |

No longer used, replaced by registrations.

---

### Not opened.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 57          | UID                                           |

No longer used, replaced by registrations.

---

### Refused.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 58          | UID                                           |

No longer used, replaced by registrations.

---

### Incorrect UID on whitelist/blacklist.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 59          | Unknown.                                      |

TODO: figure out what to do with this, because the authentication list is binary,
there are no incorrect UIDs. An incorrect type byte is still possible.

---

### Whitelist / blacklist too large.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 60          | Number of bytes too large.                    |

---

### Debug.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 62          | Anything.                                     |

Used for debug purposes, the argument can be any string.

---

### Function tag.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 63          | UID?                                          |

No longer used, should be added to registrations.

---

### Fill level ultrasonic sensor 2.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 64          | Fill level?                                   |

---

### Fill level ultrasonic sensor 3.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 65          | Fill level?                                   |

---

### Internal battery voltage without load.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 66          | Voltage (millivolts).                         |

The internal battery voltage is measured on every boot of the firmware, using the LTC4162-F battery charger.\n
The load on the battery at that time is about 40 mA.

---

### Internal battery voltage with load.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 67          | Voltage (millivolts)                          |

---

### Internal SIM ICCID.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 68          | SIM ICCID.                                    |

The internal SIM ICCID is read and stored as a status every time a connection
to the internet has been made using the modem and the internal soldered SIM.

---

### Grafix error.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 69          | Unknown.                                      |

No longer used.

---

### Temperature.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 70          | Degrees celsius.                              |

Temperature of the microcontroller's internal sensor in degrees celsius.

---

### Deposit opening stayed open.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 71          | 1 = opened, 0 = closed.                       |

Indicates if the deposit opening has stayed open after a deposit.

---

### Internal battery charge current.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 72          | Charge current in mA.                         |

Charge current of the internal battery.\n
This is only recorded when the battery is charging.

---

### Solar voltage.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 73          | Voltage (millivolts).                         |


This is only recorded when ChargerPowerSource is set to 0 for solar panel / power supply.\n
Otherwise, an external battery voltage without load status is recorded.

---

### GPS Location.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 74          | latitude,longitude                            |

The latitude and longitude are recorded in decimal degrees.\n
When the GPS location could not be detected, "Not available" is recorded instead of the location.

---

### Hardware error.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 75          | Type of error.                                |

Should be recorded when the firmware detects a hardware error.

---

### Connected cell.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 76          | PLMN, LAC hex(LAC dec), CID hex(CID dec).     |

Records the cell tower the modem is connected to.\n
The location of this cell tower can possibly be found on https://www.opencellid.org \n
The PLMN is a combination of the MCC and the MNC.

---

### Factory reset.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 77          | 1                                             |

Recorded on the first boot after a factory reset.

---

### Made active.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 78          | UID                                           |

Recorded after switching the device from inactive to active mode.\n
The UID of the card that activated the device is recorded as argument.

---

### Self test passed.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 79          | 1                                             |

Recorded after the device successfully passed a self test.

---

### Cable test passed.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 80          | 1                                             |

Recorded after the device successfully passed a cable test.

---

### Modem type.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 81          | Modem type and firmware version               |

Recorded after the modem has set up an internet connection.

---

### Registration switch error.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 82          | State of the registration inputs.             |

Recorded after a deposit has ended, and one of the registration inputs was not
at the correct level before opening the lock. The state of reach registration
input is recorded as an argument.

---

### Deposit refused.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 83          | Reason of refusal.                            |

Recorded when the Accello has refused a deposit.
The reason why the deposit has been refused is recorded as the argument.

---

### Door stayed open.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 84          | 1 for opened, 0 for closed.                   |

Recorded with a value of "1" when the Accello detects a high level on IN4,
and IN4 is still high after the number of seconds specified by the
CheckDoorStayOpen setting have expired.

Recorded with a value of "0" if a low level on IN4 is detected, and previously
this status has been recorded with a value of "1".

---

### Acceleration detect.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 85          |                                               |

Recorded after the 6D sensor signals an acceleration wakeup event.

---

### Rotation detect.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 86          | x y z                                         |

After the 6D sensor signals an acceleration wakeup event, a rotation measurement
is done for some time. The angular velocity from the 6D sensor is integrated
into rotation. When the rotation is more than a set value, it's assumed that
the container is being emptied.

---

### PCB Version.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 87          | PCB version number                            |

The PCB version is set by resistors on the PCB.

---

### IP Almere.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 100         | Unknown.                                      |

Will never be implemented.

---

### Restart Almere.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 101         | Unknown.                                      |

Will never be implemented.

---

### Deposit opening opened.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 102         | Unknown.                                      |

Will never be implemented.

---

### Deposit opening blocked.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| Yes         | 103         | Number of times unlocked but not opened.      |

The Accello keeps an internal counter that is incremented each time the
deposit opening is unlocked, but not opened. When this counter reaches
the value of the BreakCounter setting, this status is written with the number
of times the deposit opening did not open.
After that the server is contacted, and the counter is reset to 0.

The counter is also decremented after each successful deposit, but only if it's
higher than 0.

---

### IsAlive.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 104         | Unknown.                                      |

Will never be implemented.

---

### Application error Almere.

| Implemented | Status code | Argument                                      |
|-------------|-------------|-----------------------------------------------|
| No          | 105         | Unknown.                                      |

Will never be implemented.
