#ifndef ADC_H
#define ADC_H

#include <stm32f4xx.h>
#include <FreeRTOS.h>
#include <semphr.h>

/*! Interface class for the ADC interrupt handler.
    Classes that derive from this must clear the ADC_IT_EOC bit,
    either by reading the ADC data register, or by clearing the bit directly.*/
class CAdcInterrupt
{
  public:
    virtual ~CAdcInterrupt() {}

    virtual void AdcIRQHandler() = 0;
};

class CAdc : public CAdcInterrupt
{
  public:
    CAdc();
    ~CAdc();

    void              Setup(ADC_TypeDef* adc);

    float             Read(uint8_t channel);

    float             ReadVrefint();
    float             ReadVCC();
    float             ReadTempSensor();
    float             ReadVbat();

    //Cast operator to a SemaphoreHandle_t reference so that CLock(*this); can be used.
    operator          SemaphoreHandle_t&() { return m_mutex; }

  private:
    uint32_t          ADCToRCC();
    void              InstallIRQHandler();
    void              RemoveIRQHandler();
    virtual void      AdcIRQHandler();

    SemaphoreHandle_t m_mutex;

    ADC_TypeDef*      m_adc;
    volatile uint32_t m_value;
    volatile uint32_t m_numsamples;
    uint32_t          m_maxsamples;
    SemaphoreHandle_t m_done;
};

#endif //ADC_H
