#ifndef ATCMD_H
#define ATCMD_H

#include "usart.h"

enum EATCmdResult
{
  ATCmdResult_Success,
  ATCmdResult_Reply_Not_OK,
  ATCmdResult_Write_Timeout,
  ATCmdResult_Echo_Timeout,
  ATCmdResult_Echo_Usart_Error,
  ATCmdResult_Echo_Fail,
  ATCmdResult_Cmd_Too_Large,
  ATCmdResult_Read_Timeout,
  ATCmdResult_Read_Usart_Error,
  ATCmdResult_Reply_Too_Large,
  ATCmdResult_Missing_Start_Quote,
  ATCmdResult_Reply_Too_Short,
  ATCmdResult_Reply_Not_CONNECT,
};

//! Default timeout waiting for the usart to finish writing a command to the modem.
#define DEFAULT_WRITE_TIMEOUT 10000

//! Default timeout waiting for the modem echo in milliseconds.
#define DEFAULT_ECHO_TIMEOUT   1000

//! Default timeout waiting for the modem reply in milliseconds.
#define DEFAULT_READ_TIMEOUT  10000

//! Default buffer for commands and reading.
#define AT_BUFSIZE             1024

class CATCmd
{
  public:
    CATCmd(CUsart& usart);
    void ResetState();

    //AT commands, sorted alphabetically
    EATCmdResult AT();
    EATCmdResult ATCFUN(uint32_t fun, uint32_t rst);
    EATCmdResult ATCGATT_Query();
    EATCmdResult ATCGATT_Set(uint32_t state);
    EATCmdResult ATCGDCONT(uint32_t cid, const char* PDP_type = nullptr, const char* APN = nullptr,
                           const char* PDP_addr = nullptr, uint32_t* data_comp = nullptr, uint32_t* head_comp = nullptr);
    EATCmdResult ATCIMI();
    EATCmdResult ATCMEE(uint32_t format);
    EATCmdResult ATCOPN();
    EATCmdResult ATCOPS_Query();
    EATCmdResult ATCOPS_Set(uint32_t mode, const uint32_t* format = nullptr, const uint32_t* oper = nullptr, const uint32_t* act = nullptr);
    EATCmdResult ATCOPS_Set_Query();
    EATCmdResult ATCPIN_Query();
    EATCmdResult ATCPIN_Set(const char* pin);
    EATCmdResult ATCPOL_Query();
    EATCmdResult ATCREG_Query();
    EATCmdResult ATCREG_Set(uint32_t n);
    EATCmdResult ATCRSM(uint32_t command, const uint32_t* fileid = nullptr, const uint32_t* p = nullptr,
                        const char* data = nullptr, const char* pathid = nullptr);
    EATCmdResult ATCRSM_ReadBinary(uint32_t location, uint32_t length, const char** dataptr, uint32_t* datalen);
    EATCmdResult ATCSQ();
    EATCmdResult ATF();
    EATCmdResult ATGMI();
    EATCmdResult ATGMM();
    EATCmdResult ATGMR();
    EATCmdResult ATGSN();
    EATCmdResult ATI();
    EATCmdResult ATIFC(uint32_t dce_by_dte, uint32_t dte_by_dce);
    EATCmdResult ATIPR(uint32_t baudrate);
    EATCmdResult ATQAPPVER();
    EATCmdResult ATQCCID();
    EATCmdResult ATQCFG(const char* cfg);
    EATCmdResult ATQCSQ();
    EATCmdResult ATQENG(const char* cell);
    EATCmdResult ATQFOTADL(const char* url);
    EATCmdResult ATQFOTADL_State(uint32_t timeout);
    EATCmdResult ATQGDCNT_Query();
    EATCmdResult ATQGPS_Query();
    EATCmdResult ATQGPS_Set_BG96(uint32_t gnssmode, uint32_t* fixmaxtime = nullptr, uint32_t* fixmaxdist = nullptr,
                                 uint32_t* fixcount = nullptr, uint32_t* fixrate = nullptr);
    EATCmdResult ATQGPS_Set_BG95(uint32_t gnssmode, uint32_t* accuracy = nullptr, uint32_t* fixcount = nullptr,
                                 uint32_t* fixrate = nullptr, uint32_t* hepe = nullptr);
    EATCmdResult ATQGPSCFG_Query(const char* setting);
    EATCmdResult ATQGPSCFG_Set(const char* cfg);
    EATCmdResult ATQGPSDEL(uint32_t deletetype);
    EATCmdResult ATQGPSLOC(uint32_t mode);
    EATCmdResult ATQGPSEND();
    EATCmdResult ATQGPSXTRA(uint32_t enable);
    EATCmdResult ATQHTTPCFG(const char* parameter, uint32_t value);
    EATCmdResult ATQHTTPGET(uint32_t rsptime);
    EATCmdResult ATQHTTPPOST(uint32_t data_length, uint32_t input_time = 60, uint32_t rsptime = 60);
    EATCmdResult ATQHTTPPOST_Finish(uint32_t timeout);
    EATCmdResult ATQHTTPREAD_Finish();
    EATCmdResult ATQHTTPREAD_Start(uint32_t wait_time);
    EATCmdResult ATQHTTPURL(const char* url);
    EATCmdResult ATQIACT_Query();
    EATCmdResult ATQIACT_Set(uint32_t contextid);
    EATCmdResult ATQICSGP_Query(uint32_t contextid);
    EATCmdResult ATQICSGP_Set(uint32_t contextid, const uint32_t* context_type = nullptr, const char* apn = nullptr,
                              const char* username = nullptr, const char* password = nullptr, const uint32_t* authentication = nullptr);
    EATCmdResult ATQIDEACT(uint32_t contextid);
    EATCmdResult ATQINISTAT();
    EATCmdResult ATQNWINFO();
    EATCmdResult ATQNWLOCK(uint32_t* action = nullptr, uint32_t* earfcn = nullptr, uint32_t* pci = nullptr);
    EATCmdResult ATQPINC(const char* facility);
    EATCmdResult ATQPOWD(uint32_t powerstate);
    EATCmdResult ATQPRTPARA();
    EATCmdResult ATQRFTEST(const char* band, uint32_t channel, bool tx_enable, uint32_t tx_burst_or_rgi, uint32_t tx_gain_or_waveform);
    EATCmdResult ATQRFTESTMODE(uint32_t mode);
    EATCmdResult ATQSIMDET(uint32_t enable, uint32_t insertlevel);
    EATCmdResult ATQSPN();
    EATCmdResult ATQSSLCFG(const char* parameter, uint32_t sslctxID, const char* value);
    EATCmdResult ATQSSLCFG(const char* parameter, uint32_t sslctxID, uint32_t value);
    EATCmdResult ATQSSLRECV(uint32_t clientid, uint32_t readlen);

    void         SetEchoTimeout(uint32_t echotimeout)   { m_echotimeout  = echotimeout;           }
    void         ResetEchoTimeout()                     { m_echotimeout  = DEFAULT_ECHO_TIMEOUT;  }
    void         SetWriteTimeout(uint32_t writetimeout) { m_writetimeout = writetimeout;          }
    void         ResetWriteTimeout()                    { m_writetimeout = DEFAULT_WRITE_TIMEOUT; }
    void         SetReadTimeout(uint32_t readtimeout)   { m_readtimeout  = readtimeout;           }
    void         ResetReadTimeout()                     { m_readtimeout  = DEFAULT_READ_TIMEOUT;  }

    void         PrintError();

    const char*  Reply()                                { return m_readbuf;                       }
    uint32_t     ReplyLen()                             { return m_readlen;                       }
    uint32_t     ReplyNumLines();
    const char*  ReplyLine(uint32_t line, uint32_t* linelength);

    EATCmdResult ReadUsart(uint32_t len, uint32_t timeout);
    EATCmdResult WriteUsart(uint8_t* buf, uint32_t len, uint32_t timeout);

    uint8_t*     GetReadBuf()                           { return (uint8_t*)m_readbuf;             }
    uint32_t     GetReadBufSize()                       { return sizeof(m_readbuf);               }

  private:
    EATCmdResult SendCmd(const char* cmd, ...) __attribute__ ((format (printf, 2, 3)));
    EATCmdResult ReadReply(const char* start = nullptr, const char* end = nullptr);
    EATCmdResult CheckOKReply();
    EATCmdResult CheckConnectReply();
    EATCmdResult TranslateUsartResult(uint32_t result);

    CUsart&      m_usart;

    uint32_t     m_writetimeout;
    uint32_t     m_echotimeout;
    uint32_t     m_readtimeout;

    char         m_cmdbuf[AT_BUFSIZE];
    uint32_t     m_cmdlen;

    char         m_readbuf[AT_BUFSIZE];
    uint32_t     m_readlen;

    EATCmdResult m_result;
    uint32_t     m_usartresult;
};

#endif //ATCMD_H
