#ifndef CBORTOOLS_H
#define CBORTOOLS_H

#include "cbor.h"
#include "inclstdint.h"

enum CBOROption
{
  CBOROption_None             = 0x00,
  //! Don't print a log line when an element is not found.
  CBOROption_PrintNotFoundOff = 0x01,
  //! Find the last element with a specific key in a map, instead of the first.
  CBOROption_FindLast         = 0x02,
  //! If the value belonging to a key starts with a tag, skip over the tag.
  CBOROption_SkipTag          = 0x04,
};

/*! Overload for the bitwise OR | operator for enum CBOROption.
    This way, values of CBOROptions can be bitwise or-ed together
    without getting a compile error.*/
static inline CBOROption operator | (CBOROption a, CBOROption b)
{
  return (CBOROption)((uint32_t)a | (uint32_t)b);
}

bool CBORFindMapValue(uint32_t loglevel, const char* rootname, CborValue* root, const char* name, CborValue* value, CBOROption options = CBOROption_None);
bool CBORFindMapString(uint32_t loglevel, const char* rootname, CborValue* root, const char* name, CborValue* value, CBOROption options = CBOROption_None);
bool CBORFindMapArray(uint32_t loglevel, const char* rootname, CborValue* root, const char* name, CborValue* value);
bool CBORFindMapInt(uint32_t loglevel, const char* rootname, CborValue* root, const char* name, int64_t* value, CBOROption options = CBOROption_None);
bool CBORFindIntMapValue(uint32_t loglevel, const char* rootname, CborValue* root, int64_t key, CborValue* value, CBOROption options = CBOROption_None);
bool CBORFindIntMapInt(uint32_t loglevel, const char* rootname, CborValue* root, int64_t key, int64_t* value, CBOROption options = CBOROption_None);
bool CBORFindIntMapString(uint32_t loglevel, const char* rootname, CborValue* root, int64_t key, CborValue* value, CBOROption options = CBOROption_None);
bool CBORFindIntMapBool(uint32_t loglevel, const char* rootname, CborValue* root, int64_t key, bool* value, CBOROption options = CBOROption_None);
bool CBORFindIntMapByteString(uint32_t loglevel, const char* rootname, CborValue* root, int64_t key, CborValue* value, CBOROption options = CBOROption_None);
bool CBORFindIntMapFloat(uint32_t loglevel, const char* rootname, CborValue* root, int64_t key, float* value, CBOROption options = CBOROption_None);
bool CBORCompareString(uint32_t loglevel, const char* name, CborValue* element, const char* str, bool* result);
bool CBORGetStringLength(uint32_t loglevel, const char* name, CborValue* element, uint32_t* length);
bool CBORCopyString(uint32_t loglevel, const char* name, CborValue* element, char* buf, uint32_t* length);
bool CBORCopyByteString(uint32_t loglevel, const char* name, CborValue* element, uint8_t* buf, uint32_t* length);
bool CBOREnterContainer(uint32_t loglevel, const char* name, CborValue* container, CborValue* it);
bool CBORLeaveContainer(uint32_t loglevel, const char* name, CborValue* container, CborValue* it);
bool CBORValueAdvance(uint32_t loglevel, const char* name, CborValue* it);
bool CBORCloseContainer(uint32_t loglevel, const char* name, CborEncoder* encoder, CborEncoder* container);
bool CBORSetRTCFromDateField(uint32_t loglevel, const char* rootname, CborValue* root, const char* name);
bool CBORPrint(uint32_t loglevel, const char* prefix, uint8_t* buf, uint32_t bufsize);

#endif //CBORTOOLS_H
