#ifndef DMA_H
#define DMA_H

#include <stm32f4xx.h>

#include <FreeRTOS.h>
#include <event_groups.h>

#include "inclstdint.h"

/*! Objects that want to use an interrupt of a DMA stream should derive from this class,
    then call g_dma.EnableIrq(stream, this, priority) and enable the wanted interrupt on the stream,
    the DMAIRQHandler() virtual method is called when the interrupt fires.*/
class CDmaIrq
{
  public:
    virtual void DMAIRQHandler() = 0;
};

class CDma
{
  public:
    CDma();

    void               Setup();

    void               LockStream(DMA_Stream_TypeDef* stream);
    void               UnlockStream(DMA_Stream_TypeDef* stream);

    void               EnableIrq(DMA_Stream_TypeDef* stream, CDmaIrq* irqhandler, uint8_t priority);
    void               DisableIrq(DMA_Stream_TypeDef* stream);

  private:
    uint32_t           DMABit(DMA_Stream_TypeDef* stream);
    IRQn_Type          DMAIRQn(DMA_Stream_TypeDef* stream);
    uint32_t           DMANum(DMA_Stream_TypeDef* stream);

    EventGroupHandle_t m_streamlock;
};

extern CDma g_dma;

#endif //DMA_H
