#ifndef EXTI_H
#define EXTI_H

// #include <gd32f4xx.h>
#include <stm32f4xx.h>

#include <FreeRTOS.h>

#include "gpiopin.h"
#include "gpio.h"
#include "inclstdint.h"

class CGPIOInterrupt
{
  public:
    virtual ~CGPIOInterrupt() {}

    virtual void GPIOIRQHandler(SGPIOPin pin) = 0;
};

enum ERTCIRQType
{
  RTCIRQAlarmA = 0,
  RTCIRQAlarmB = 1,
  RTCIRQWakeup = 2,
  NumRTCIRQs
};

class CRTCInterrupt
{
  public:
    virtual ~CRTCInterrupt() {}

    virtual void RTCIRQHandler(ERTCIRQType irqtype) = 0;
};

class CExti
{
  public:
    CExti();

    void           EnableGPIOIrq(SGPIOPin pin, EXTITrigger_TypeDef type, CGPIOInterrupt* irqhandler);
    void           DisableGPIOIrq(SGPIOPin pin);

    void           EnableRTCIrq(ERTCIRQType type, CRTCInterrupt* irqhandler);
    void           DisableRTCIrq(ERTCIRQType type);

    void           OnGPIOIrq(uint32_t pinnum);
    void           OnRTCIrq(ERTCIRQType type);

  private:
    uint32_t       ExtiLineByNum(uint32_t num);
    uint32_t       ExtiLineByRTC(ERTCIRQType irqtype);
    IRQn           ExtiIrqByNum(uint32_t num);
    IRQn           ExtiIrqByRTC(ERTCIRQType irqtype);
    void           SetupInterrupt(uint32_t pinnum, bool enable, uint8_t priority = configLIBRARY_MAX_SYSCALL_INTERRUPT_PRIORITY);
    void           SetSharedInterrupt(uint32_t start, uint32_t end, uint32_t pinnum, bool enable, uint8_t priority);

    struct
    {
      SGPIOPin        pin;
      CGPIOInterrupt* irqhandler;
    }
    m_gpioirqhandlers[PINS_PER_PORT]; //!< Array of installed irq handlers, irqhandler is NULL when not installed.

    CRTCInterrupt* m_rtcirqhandlers[NumRTCIRQs];
};

extern CExti g_exti;

#endif //EXTI_H
