#ifndef FWIMAGEPARAMS_H
#define FWIMAGEPARAMS_H

#define FW_IV_SIZE        16  //!< Size of the AES-CBC initialization vector.
#define FW_BLOCK_SIZE     512 //!< Number of data bytes per encrypted block.
#define FW_BLOCKNUM_SIZE  16  //!< Size of the block number.
#define FW_HASH_SIZE      32  //!< Size of the SHA256 hash in bytes.

//! Total size of an encrypted block.
#define FW_ENCRYPTED_BLOCK_SIZE (FW_IV_SIZE + FW_BLOCK_SIZE + FW_BLOCKNUM_SIZE + FW_HASH_SIZE)

//! Definition of an encrypted block.
struct __attribute__((packed)) SEncryptedBlock
{
  //always zero initialize all storage members
  SEncryptedBlock() : iv {}, block {}, blocknum {}, hash {}
  {
  }

  uint8_t  iv[FW_IV_SIZE];
  uint8_t  block[FW_BLOCK_SIZE];
  uint8_t  blocknum[FW_BLOCKNUM_SIZE];
  uint8_t  hash[FW_HASH_SIZE];
};

#define FW_SIGNATURE_SIZE      (FW_BLOCK_SIZE)     //!< Size of the signature in bytes.
#define FW_HEADER_SIZE         (FW_BLOCK_SIZE)     //!< Size of the header in bytes.

#define FW_FILENAME_MAX_LENGTH 32                  //!< Maximum length of a filename.

#define FW_FILES               0                   //!< Indicates the files array in the root map.
#define FW_FILENAME            1                   //!< Indicates the filename in the file map.

/*!< Indicates the position of the file in the file map.
     Files should start at a position of a multiple of FW_BLOCK_SIZE.*/
#define FW_FILEPOSITION        2

#define FW_FILESIZE            3                   //!< Indicates the size of the file in the file map.
#define FW_FILEHASH            4                   //!< Indicates the SHA256 of the file in the file map.

/*! Value for the version field in the signature.
    If the format for a firmware update image changes significantly,
    the version can be increased so that the firmware can detect this change.*/
#define FW_SIGNATURE_VERSION   1

struct __attribute__((packed)) SSignature
{
  //always zero initialize all member variables
  SSignature() : version(0), size(0), signature {}
  {
  }

  uint32_t version;
  uint32_t size;
  uint8_t  signature[FW_HASH_SIZE];
};

#endif //FWIMAGEPARAMS_H
