#ifndef GPIO_H
#define GPIO_H

#include <stm32f4xx.h>

#define NUM_PORTS     11
#define PINS_PER_PORT 16

/*! Describes the state of a GPIO. Used in GPIOInit().*/
typedef struct
{
  BitAction        state; /*!< Output state of the pin, HIGH or LOW.*/
  GPIO_InitTypeDef init;  /*!< Initialization state for GPIO_Init().*/
  int32_t          af;    /*!< Alternate function, used for the GPIO_AF defines in stm32f4xx_gpio.h, set to -1 to disable.*/
} SGPIOStateInit;

#define HIGH Bit_SET
#define LOW  Bit_RESET

void          GPIOInit();
void          GPIOInitPin(GPIO_TypeDef* port, uint32_t pin,
                          const SGPIOStateInit* stateinit = nullptr);
void          GPIOInitPinNum(uint32_t port, uint32_t pin,
                             const SGPIOStateInit* stateinit = nullptr);
GPIO_TypeDef* GPIOPortByNum(uint32_t portnum);
uint16_t      GPIOPinByNum(uint32_t pinnum);
uint32_t      GPIONumByPin(uint16_t pin);
uint32_t      GPIONumByPort(GPIO_TypeDef* port);
uint32_t      EXTIPortSourceByPort(GPIO_TypeDef* port);
uint8_t       WakeupPinBootState();

#endif //GPIO_H
