#ifndef GPIOPIN_H
#define GPIOPIN_H

#include <stm32f4xx.h>

#define WAKEUP_PORT GPIOA
#define WAKEUP_PIN  GPIO_Pin_0
#define WAKEUP      ((const struct SGPIOPin){WAKEUP_PORT, WAKEUP_PIN})

struct SGPIOPin
{
  GPIO_TypeDef* port;
  uint16_t      pin;
};

static inline void GPIO_Set(struct SGPIOPin pin)
{
  GPIO_SetBits(pin.port, pin.pin);
}

static inline void GPIO_Reset(struct SGPIOPin pin)
{
  GPIO_ResetBits(pin.port, pin.pin);
}

static inline void GPIO_Toggle(struct SGPIOPin pin)
{
  GPIO_ToggleBits(pin.port, pin.pin);
}

/*! Sets a GPIO pin to the value specified by value.
    \param pin The pin to set.
    \param value Can be Bit_SET or Bit_RESET, or HIGH or LOW from gpio.h*/
static inline void GPIO_WritePin(struct SGPIOPin pin, BitAction value)
{
  GPIO_WriteBit(pin.port, pin.pin, value);
}

#ifdef __cplusplus
/*! Overload so that true and false can be used for the pin state.*/
static inline void GPIO_WritePin(struct SGPIOPin pin, bool value)
{
  GPIO_WritePin(pin, value ? Bit_SET : Bit_RESET);
}
#endif

static inline uint8_t GPIO_ReadInput(struct SGPIOPin pin)
{
  return GPIO_ReadInputDataBit(pin.port, pin.pin);
}

#endif //GPIOPIN_H
