#ifndef I2CPERIPHERAL_H
#define I2CPERIPHERAL_H

#include <stm32f4xx.h>
#include <FreeRTOS.h>
#include <semphr.h>
#include "gpiopin.h"

#define DEFAULT_I2C_CLOCK_SPEED 100000

class CI2C
{
  public:
    CI2C();
    ~CI2C();

    void              Setup(I2C_TypeDef* i2c, SGPIOPin sclpin, uint32_t clockspeed = DEFAULT_I2C_CLOCK_SPEED);
    void              Close();

    bool              Write(uint8_t address, const uint8_t* buf1, uint32_t size1, const uint8_t* buf2 = nullptr, uint32_t size2 = 0);

    /*! Template function to pass a const array by reference, so that the sizeof operator can be used on it.
        example: m_i2c.Write(addr, (const uint8_t[]){0x00, 0xAE});
        \param address the I2C address to write to.
        \param buf A reference to a const uint8_t array.
        \return true if the write succeeded, false if it failed.*/
    template<typename T>
    bool              Write(uint8_t address, const T& buf)
                      { return Write(address, buf, (uint32_t)sizeof(buf), nullptr, (uint32_t)0); }

    bool              Read(uint8_t address, uint8_t* buf, uint32_t size);

    void              Stop();

    /*! \return Which byte failed to ack at a Write() or Read() call.
                Byte 0 is the address, byte 1 is the first data byte etc.*/
    uint32_t          AckFail() { return m_ackfail; }

    //cast operator to a SemaphoreHandle_t reference so that CLock(*this); can be used
    operator          SemaphoreHandle_t&() { return m_mutex; }

    const char*       Name();

  private:
    void              SendReset(SGPIOPin sclpin);
    void              SetClock(FunctionalState NewState);
    bool              CheckAck();
    I2C_TypeDef*      m_i2c;
    uint32_t          m_ackfail;

    SemaphoreHandle_t m_mutex;
};

#endif //I2CPERIPHERAL_H
