#ifndef IOEXPANDER_H
#define IOEXPANDER_H

#include "i2c.h"
#include "inclstdint.h"

#define IOEXP_GP0 (1 << 0)
#define IOEXP_GP1 (1 << 1)
#define IOEXP_GP2 (1 << 2)
#define IOEXP_GP3 (1 << 3)
#define IOEXP_GP4 (1 << 4)
#define IOEXP_GP5 (1 << 5)
#define IOEXP_GP6 (1 << 6)
#define IOEXP_GP7 (1 << 7)

#define IOEXP_ALL (0xFF)

/*! Interface class for the MCP23008 IO expander.*/
class CIOExpander
{
  public:
    CIOExpander();
    ~CIOExpander();

    bool        Setup(CI2C* i2c, uint8_t addr, const char* name);
    bool        ResetToDefaults();
    bool        SetAsInput(uint8_t pins);
    bool        SetAsOutput(uint8_t pins);
    bool        Read(uint8_t pin, bool* value);
    bool        ReadGPIO(uint8_t* gpio);
    bool        Set(uint8_t pins);
    bool        Reset(uint8_t pins);
    bool        SetInputPolarity(uint8_t pins, bool inverted);
    bool        EnableInterrupt(uint8_t pins);
    bool        DisableInterrupt(uint8_t pins);
    bool        SetDefaultCompare(uint8_t pins, bool value);
    bool        SetInterruptOnChange(uint8_t pins);
    bool        SetInterruptOnNotDefault(uint8_t pins);
    bool        EnablePullup(uint8_t pins);
    bool        DisablePullup(uint8_t pins);
    bool        GetInterruptFlag(uint8_t pin, bool* value);
    bool        GetInterruptCapture(uint8_t* value);
    bool        SetOutputPolarity(bool inverted);

  private:
    bool        ReadRegister(uint8_t reg, uint8_t* value);
    bool        WriteRegister(uint8_t reg, uint8_t value);
    bool        SetPinsInReg(uint8_t reg, uint8_t pins, bool value, const char* regname);

    CI2C*       m_i2c;
    uint8_t     m_addr;
    const char* m_name;
};

#endif //IOEXPANDER_H
