#ifndef RFM9xW_H
#define RFM9xW_H

#include <stm32f4xx.h>
#include "gpiopin.h"
#include "spi.h"
#include "exti.h"
#include "rfm9xwconfig.h"

#define RFM9xW_FIFO_SIZE 256

enum ECheckWrittenValue
{
  NoCheckWrittenValue,
  CheckWrittenValue,
};

enum EModulation
{
  Modulation_LoRa,
  Modulation_OOK,
};

class CRFM9xWInterrupt
{
  public:
    virtual ~CRFM9xWInterrupt() {}

    virtual void CRFM9xWInterruptHandler() = 0;
};

class CRFM9xW : public CGPIOInterrupt
{
  public:
    CRFM9xW();
    ~CRFM9xW();

    bool              Setup(const char* devicestr, SPI_TypeDef* spi, SGPIOPin nss, SGPIOPin rst,
                            SGPIOPin dio0, CRFM9xWInterrupt* irqhandler);
    void              Close();

    uint8_t           ReadRegister(uint8_t address);
    bool              WriteRegister(uint8_t address, uint8_t value, ECheckWrittenValue check = CheckWrittenValue);

    bool              CheckVersion();
    bool              CheckOperation(EMode mode);
    uint32_t          NextCheckDelay();

    bool              SetMode(enum EMode mode);
    EMode             GetMode();
    bool              SetModulation(EModulation modulation);
    bool              SetFrequency(uint32_t frequency);
    uint32_t          GetFrequency();
    bool              SetLowFrequencyMode(bool on);
    bool              GetLowFrequencyMode();
    bool              SetTransmitPower(uint32_t dbm);

    bool              SetLoRaLnaAgc(bool enable);
    bool              SetLoRaBandwidth(ELoRaSignalBandwidth bandwidth);
    bool              SetLoRaCodingRate(ELoRaCodingRate codingrate);
    bool              SetLoRaHeaderMode(ELoRaHeaderMode headermode);
    bool              SetLoRaCrcMode(ELoRaCrcMode crcmode);
    bool              SetLoRaSpreadingFactor(ELoRaSpreadingFactor spreadingfactor);
    bool              SetLoRaBaseAddresses(uint8_t txbaseaddr, uint8_t rxbaseaddr);
    float             GetLoRaPacketRssi();
    bool              SetLoRaDIO0Mode(EDIO0Mode dio0mode);
    bool              GetLoRaCrcOnPayload();
    uint8_t           GetLoRaPacketSize();
    uint8_t           GetLoRaPacketAddress();
    bool              SetLoRaFifoPtr(uint8_t fifoptr);
    void              ReadLoRaFifo(uint8_t* buf, uint8_t size);
    bool              SetLoRaPacketSize(uint8_t packetsize);
    void              WriteLoRaFifo(uint8_t* buf, uint8_t size);
    bool              GetLoRaIsReceiving();

    bool              SetOOKLnaAgc(bool enable);
    bool              SetOOKBitSynchronizer(EBitSynchronizer bitsynchronizer);
    bool              SetOOKThresholdMode(EOOKThreshType threshmode);
    bool              SetOOKFixedThreshold(uint8_t threshold);
    bool              SetOOKPeakThreshDec(EOOKPeakThreshDec decr);
    bool              SetOOKDataMode(EOOKDataMode datamode);
    bool              SetOOKRxBw(ERxBw bw);
    bool              SetOOKRssiSmoothing(ERssiSmoothing rssismoothing);
    float             GetOOKRssi();

    void              EnableDIO0Interrupt();
    void              DisableDIO0Interrupt();

    uint8_t           GetIRQFlags();

  private:
    void              ResetState();
    void              Select();
    void              Deselect();
    virtual void      GPIOIRQHandler(SGPIOPin pin);
    
    bool              m_activated;
    uint32_t          m_lastcheck;
    const char*       m_devicestr;

    SGPIOPin          m_nss;
    SGPIOPin          m_rst;
    SGPIOPin          m_dio0;

    CSpi              m_spi;

    CRFM9xWInterrupt* m_irqhandler;

    uint32_t          m_frequency;
};

#endif //RFM9xW_H
