#ifndef RFM9xWCONFIG_H
#define RFM9xWCONFIG_H

#include <cassert>

//! Bit to set in the first byte to indicate a write to the register.
#define WRITE_CMD                         0x80

//registers
#define REG_FIFO                          0x00
#define REG_OP_MODE                       0x01
#define REG_BITRATE_MSB                   0x02
#define REG_BITRATE_LSB                   0x03
#define REG_FDEV_MSB                      0x04
#define REG_FDEV_LSB                      0x05
#define REG_FRF_MSB                       0x06
#define REG_FRF_MID                       0x07
#define REG_FRF_LSB                       0x08
#define REG_PA_CONFIG                     0x09
#define REG_PA_RAMP                       0x0a
#define REG_OCP                           0x0b
#define REG_LNA                           0x0c
#define REG_RX_CONFIG                     0x0d
#define REG_FIFO_ADDR_PTR                 0x0d
#define REG_RSSI_CONFIG                   0x0e
#define REG_FIFO_TX_BASE_ADDR             0x0e
#define REG_RSSI_COLLISION                0x0f
#define REG_FIFO_RX_BASE_ADDR             0x0f
#define REG_RSSI_THRESH                   0x10
#define REG_FIFO_RX_CURRENT_ADDR          0x10
#define REG_RSSI_VALUE                    0x11
#define REG_IRQ_FLAGS_MASK                0x11
#define REG_RX_BW                         0x12
#define REG_IRQ_FLAGS                     0x12
#define REG_AFC_BW                        0x13
#define REG_RX_NB_BYTES                   0x13
#define REG_OOK_PEAK                      0x14
#define REG_RX_HEADER_CNT_VALUE_MSB       0x14
#define REG_OOK_FIX                       0x15
#define REG_RX_HEADER_CNT_VALUE_LSB       0x15
#define REG_OOK_AVG                       0x16
#define REG_RX_PACKET_CNT_VALUE_MSB       0x16
#define REG_RX_PACKET_CNT_VALUE_LSB       0x17
#define REG_MODEM_STAT                    0x18
#define REG_PKT_SNR_VALUE                 0x19
#define REG_AFC_FEI                       0x1a
#define REG_PKT_RSSI_VALUE                0x1a
#define REG_AFC_MSB                       0x1b
#define REG_RSSI_VALUE_LORA               0x1b
#define REG_AFC_LSB                       0x1c
#define REG_HOP_CHANNEL                   0x1c
#define REG_FEI_MSB                       0x1d
#define REG_MODEM_CONFIG_1                0x1d
#define REG_FEI_LSB                       0x1e
#define REG_MODEM_CONFIG_2                0x1e
#define REG_PREAMBLE_DETECT               0x1f
#define REG_SYMB_TIMEOUT_LSB              0x1f
#define REG_RX_TIMEOUT_1                  0x20
#define REG_PREAMBLE_MSB_LORA             0x20
#define REG_RX_TIMEOUT_2                  0x21
#define REG_PREAMBLE_LSB_LORA             0x21
#define REG_RX_TIMEOUT_3                  0x22
#define REG_PAYLOAD_LENGTH_LORA           0x22
#define REG_RX_DELAY                      0x23
#define REG_MAX_PAYLOAD_LENGTH            0x23
#define REG_OSC                           0x24
#define REG_HOP_PERIOD                    0x24
#define REG_PREAMBLE_MSB                  0x25
#define REG_FIFO_RX_BYTE_ADDR             0x25
#define REG_PREAMBLE_LSB                  0x26
#define REG_MODEM_CONFIG_3                0x26
#define REG_SYNC_CONFIG                   0x27
#define REG_SYNC_VALUE_1                  0x28
#define REG_FEI_MSB_LORA                  0x28
#define REG_SYNC_VALUE_2                  0x29
#define REG_FEI_MID_LORA                  0x29
#define REG_SYNC_VALUE_3                  0x2a
#define REG_FEI_LSB_LORA                  0x2a
#define REG_SYNC_VALUE_4                  0x2b
#define REG_SYNC_VALUE_5                  0x2c
#define REG_RSSI_WIDEBAND                 0x2c
#define REG_SYNC_VALUE_6                  0x2d
#define REG_SYNC_VALUE_7                  0x2e
#define REG_SYNC_VALUE_8                  0x2f
#define REG_PACKET_CONFIG_1               0x30
#define REG_PACKET_CONFIG_2               0x31
#define REG_DETECTION_OPTIMIZE            0x31
#define REG_PAYLOAD_LENGTH                0x32
#define REG_NODE_ADRS                     0x33
#define REG_INVERT_IQ                     0x33
#define REG_BROADCAST_ADRS                0x34
#define REG_FIFO_THRESH                   0x35
#define REG_SEQ_CONFIG_1                  0x36
#define REG_SEQ_CONFIG_2                  0x37
#define REG_DETECTION_THRESHOLD           0x37
#define REG_TIMER_RESOL                   0x38
#define REG_TIMER_1_COEF                  0x39
#define REG_SYNC_WORD                     0x39
#define REG_TIMER_2_COEF                  0x3a
#define REG_IMAGE_CAL                     0x3b
#define REG_TEMP                          0x3c
#define REG_LOW_BAT                       0x3d
#define REG_IRQ_FLAGS_1                   0x3e
#define REG_IRQ_FLAGS_2                   0x3f
#define REG_DIO_MAPPING_1                 0x40
#define REG_DIO_MAPPING_2                 0x41
#define REG_VERSION                       0x42
#define REG_PLL_HOP                       0x44
#define REG_TCXO                          0x4b
#define REG_PA_DAC                        0x4d
#define REG_FORMER_TEMP                   0x5b
#define REG_BIT_RATE_FRAC                 0x5d
#define REG_AGC_REF                       0x61
#define REG_AGC_THRESH_1                  0x62
#define REG_AGC_THRESH_2                  0x63
#define REG_AGC_THRESH_3                  0x64
#define REG_PLL                           0x70

#define RFM9xW_VERSION                    0x12 //!< Default value of @ref REG_VERSION.

#define MAP_PREAMBLE_DETECT               0x01

#define OP_MODE_LORA                      0x80
#define OP_MODE_MODULATION_FSK            0x00
#define OP_MODE_MODULATION_OOK            0x20
#define OP_MODE_LOW_FREQUENCY_MODE_ON     0x08

#define MODE_MASK                         0x07
enum EMode
{
  ModeSleep    = 0,
  ModeStandby  = 1,
  ModeFstx     = 2,
  ModeTx       = 3,
  ModeFsrx     = 4,
  ModeRx       = 5,
  ModeRxCont   = 5,
  ModeRxSingle = 6,
  ModeCad      = 7
};

/*! Checks mode to be a valid value of @ref EMode.
    \param mode A value of EMode.*/
#define CHECK_MODE(mode) \
  assert(mode == ModeSleep    || \
         mode == ModeStandby  || \
         mode == ModeFstx     || \
         mode == ModeTx       || \
         mode == ModeFsrx     || \
         mode == ModeRx       || \
         mode == ModeRxSingle || \
         mode == ModeCad)

#define PA_BOOST                          0x80
#define OUTPUT_POWER_MASK                 0x0F

#define LORA_LNA_AGC_BIT                  0x04

#define LORA_SIGNAL_BANDWIDTH_MASK        0xF0
enum ELoRaSignalBandwidth
{
  LoRaSignalBandwidth_7_8   = (0 << 4),
  LoRaSignalBandwidth_10_4  = (1 << 4),
  LoRaSignalBandwidth_15_6  = (2 << 4),
  LoRaSignalBandwidth_20_8  = (3 << 4),
  LoRaSignalBandwidth_31_25 = (4 << 4),
  LoRaSignalBandwidth_41_7  = (5 << 4),
  LoRaSignalBandwidth_62_5  = (6 << 4),
  LoRaSignalBandwidth_125   = (7 << 4),
  LoRaSignalBandwidth_250   = (8 << 4),
  LoRaSignalBandwidth_500   = (9 << 4)
};

/*! Checks bandwidth to be a correct value of @ref ELoRaSignalBandwidth.
    \param bandwidth A value of @ref ELoRaSignalBandwidth.*/
#define CHECK_BANDWIDTH(bandwidth) \
  assert(bandwidth == LoRaSignalBandwidth_7_8   ||\
         bandwidth == LoRaSignalBandwidth_10_4  ||\
         bandwidth == LoRaSignalBandwidth_15_6  ||\
         bandwidth == LoRaSignalBandwidth_20_8  ||\
         bandwidth == LoRaSignalBandwidth_31_25 ||\
         bandwidth == LoRaSignalBandwidth_41_7  ||\
         bandwidth == LoRaSignalBandwidth_62_5  ||\
         bandwidth == LoRaSignalBandwidth_125   ||\
         bandwidth == LoRaSignalBandwidth_250   ||\
         bandwidth == LoRaSignalBandwidth_500)

#define LORA_CODING_RATE_MASK             0x0E
enum ELoRaCodingRate
{
  LoRaCodingRate_4_5 = (1 << 1),
  LoRaCodingRate_4_6 = (2 << 1),
  LoRaCodingRate_4_7 = (3 << 1),
  LoRaCodingRate_4_8 = (4 << 1)
};

/*! Checks codingrate to be a correct value of @ref ELoRaCodingRate.
    \param codinrate A value of @ref ELoRaCodingRate.*/
#define CHECK_CODING_RATE(codingrate) \
  assert(codingrate == LoRaCodingRate_4_5 ||\
         codingrate == LoRaCodingRate_4_6 ||\
         codingrate == LoRaCodingRate_4_7 ||\
         codingrate == LoRaCodingRate_4_8)

#define LORA_HEADER_MODE_MASK    0x01
enum ELoRaHeaderMode
{
  LoRaExplicitHeaderMode = 0,
  LoRaImplicitHeaderMode = 1
};

/*! Checks headermode to be a correct value of @ref ELoRaHeaderMode.
    \param headermode A value of @ref ELoRaHeaderMode.*/
#define CHECK_HEADER_MODE(headermode) \
  assert(headermode == LoRaExplicitHeaderMode || \
         headermode == LoRaImplicitHeaderMode)

#define LORA_CRC_MODE_MASK                0x04
enum ELoRaCrcMode
{
  LoRaCrcModeOff = (0 << 2),
  LoRaCrcModeOn  = (1 << 2)
};

/*! Checks crcmode to be a correct value of @ref ELoRaCrcMode.
    \param crcmode A value of @ref ELoRaCrcMode.*/
#define CHECK_CRC_MODE(crcmode) \
  assert(crcmode == LoRaCrcModeOff || \
         crcmode == LoRaCrcModeOn)

#define LORA_SPREADING_FACTOR_MASK        0xF0
enum ELoRaSpreadingFactor
{
  LoRaSpreadingFactor_6  = (6  << 4),
  LoRaSpreadingFactor_7  = (7  << 4),
  LoRaSpreadingFactor_8  = (8  << 4),
  LoRaSpreadingFactor_9  = (9  << 4),
  LoRaSpreadingFactor_10 = (10 << 4),
  LoRaSpreadingFactor_11 = (11 << 4),
  LoRaSpreadingFactor_12 = (12 << 4)
};

#define CHECK_SPREADING_FACTOR(spreadingfactor) \
  assert(spreadingfactor == LoRaSpreadingFactor_6  || \
         spreadingfactor == LoRaSpreadingFactor_7  || \
         spreadingfactor == LoRaSpreadingFactor_8  || \
         spreadingfactor == LoRaSpreadingFactor_9  || \
         spreadingfactor == LoRaSpreadingFactor_10 || \
         spreadingfactor == LoRaSpreadingFactor_11 || \
         spreadingfactor == LoRaSpreadingFactor_12)

#define IRQ_FLAG_LORA_RX_TIMEOUT          (1 << 7)
#define IRQ_FLAG_LORA_RX_DONE             (1 << 6)
#define IRQ_FLAG_LORA_PAYLOAD_CRC_ERROR   (1 << 5)
#define IRQ_FLAG_LORA_VALID_HEADER        (1 << 4)
#define IRQ_FLAG_LORA_TX_DONE             (1 << 3)
#define IRQ_FLAG_LORA_CAD_DONE            (1 << 2)
#define IRQ_FLAG_LORA_FHSS_CHANGE_CHANNEL (1 << 1)
#define IRQ_FLAG_LORA_CAD_DETECTED        (1 << 0)

#define LORA_SIGNAL_DETECTED_BIT          (1 << 0)
#define LORA_SIGNAL_SYNCHRONIZED_BIT      (1 << 1)
#define LORA_RX_ONGOING_BIT               (1 << 2)

#define OOK_LNA_AGC_BIT                   0x08

#define OOK_BIT_SYNCHRONIZER_MASK         0x20
enum EBitSynchronizer
{
  BitSynchronizer_Off = (0 << 5),
  BitSynchronizer_On  = (1 << 5),
};

#define CHECK_BIT_SYNCHRONIZER(bitsynchronizer) \
assert(bitsynchronizer == BitSynchronizer_Off || \
       bitsynchronizer == BitSynchronizer_On)

#define OOK_THRESH_TYPE_MASK              0x18
enum EOOKThreshType
{
  ThreshType_Fixed   = (0 << 3),
  ThreshType_Peak    = (1 << 3),
  ThreshType_Average = (2 << 3),
};

#define CHECK_THRESH_TYPE(thresh) \
assert(thresh == ThreshType_Fixed || \
       thresh == ThreshType_Peak  || \
       thresh == ThreshType_Average)

#define OOK_PEAK_THRESH_DEC_MASK          0xe0
enum EOOKPeakThreshDec
{
  EOOKPeakThreshDec_Once_Per_Chip          = (0 << 5),
  EOOKPeakThreshDec_Once_Every_Two_Chips   = (1 << 5),
  EOOKPeakThreshDec_Once_Every_Four_Chips  = (2 << 5),
  EOOKPeakThreshDec_Once_Every_Eight_Chips = (3 << 5),
  EOOKPeakThreshDec_Twice_Per_Chip         = (4 << 5),
  EOOKPeakThreshDec_Four_Times_Per_Chip    = (5 << 5),
  EOOKPeakThreshDec_Eight_Times_Per_Chip   = (6 << 5),
  EOOKPeakThreshDec_Sixteen_Times_Per_Chip = (7 << 5),
};

#define CHECK_OOK_PEAK_THRESH_DEC(dec) \
  assert(dec == EOOKPeakThreshDec_Once_Per_Chip          || \
         dec == EOOKPeakThreshDec_Once_Every_Two_Chips   || \
         dec == EOOKPeakThreshDec_Once_Every_Four_Chips  || \
         dec == EOOKPeakThreshDec_Once_Every_Eight_Chips || \
         dec == EOOKPeakThreshDec_Twice_Per_Chip         || \
         dec == EOOKPeakThreshDec_Four_Times_Per_Chip    || \
         dec == EOOKPeakThreshDec_Eight_Times_Per_Chip   || \
         dec == EOOKPeakThreshDec_Sixteen_Times_Per_Chip)

#define DATA_MODE_MASK                    0x40
enum EOOKDataMode
{
  OOKDataMode_Continuous = (0 << 6),
  OOKDataMode_Packet     = (1 << 6),
};

#define CHECK_DATA_MODE(datamode) \
  assert(datamode == OOKDataMode_Continuous || \
         datamode == OOKDataMode_Packet)

#define IRQ_FLAG_OOK_MODE_READY           0x80

#define RX_BW_MASK                        0x1F
enum ERxBw
{
  RxBw_2_6     = (2 << 3) | 7,
  RxBw_3_1     = (1 << 3) | 7,
  RxBw_3_9     = (0 << 3) | 7,
  RxBw_5_2     = (2 << 3) | 6,
  RxBw_6_3     = (1 << 3) | 6,
  RxBw_7_8     = (0 << 3) | 6,
  RxBw_10_4    = (2 << 3) | 5,
  RxBw_12_5    = (1 << 3) | 5,
  RxBw_15_6    = (0 << 3) | 5,
  RxBw_20_8    = (2 << 3) | 4,
  RxBw_25_0    = (1 << 3) | 4,
  RxBw_31_3    = (0 << 3) | 4,
  RxBw_41_7    = (2 << 3) | 3,
  RxBw_50_0    = (1 << 3) | 3,
  RxBw_62_5    = (0 << 3) | 3,
  RxBw_83_3    = (2 << 3) | 2,
  RxBw_100_0   = (1 << 3) | 2,
  RxBw_125_0   = (0 << 3) | 2,
  RxBw_166_7   = (2 << 3) | 1,
  RxBw_200_0   = (1 << 3) | 1,
  RxBw_250_0   = (0 << 3) | 1,
};

#define CHECK_RX_BW(bw) \
assert(bw == RxBw_2_6   || \
       bw == RxBw_3_1   || \
       bw == RxBw_3_9   || \
       bw == RxBw_5_2   || \
       bw == RxBw_6_3   || \
       bw == RxBw_7_8   || \
       bw == RxBw_10_4  || \
       bw == RxBw_12_5  || \
       bw == RxBw_15_6  || \
       bw == RxBw_20_8  || \
       bw == RxBw_25_0  || \
       bw == RxBw_31_3  || \
       bw == RxBw_41_7  || \
       bw == RxBw_50_0  || \
       bw == RxBw_62_5  || \
       bw == RxBw_83_3  || \
       bw == RxBw_100_0 || \
       bw == RxBw_125_0 || \
       bw == RxBw_166_7 || \
       bw == RxBw_200_0 || \
       bw == RxBw_250_0)

#define RSSI_SMOOTHING_MASK               0x07
enum ERssiSmoothing
{
  RssiSmoothing_2   = 0,
  RssiSmoothing_4   = 1,
  RssiSmoothing_8   = 2,
  RssiSmoothing_16  = 3,
  RssiSmoothing_32  = 4,
  RssiSmoothing_64  = 5,
  RssiSmoothing_128 = 6,
  RssiSmoothing_256 = 7,
};

#define CHECK_RSSI_SMOOTHING(smoothing) \
assert(smoothing == RssiSmoothing_2   || \
       smoothing == RssiSmoothing_4   || \
       smoothing == RssiSmoothing_8   || \
       smoothing == RssiSmoothing_16  || \
       smoothing == RssiSmoothing_32  || \
       smoothing == RssiSmoothing_64  || \
       smoothing == RssiSmoothing_128 || \
       smoothing == RssiSmoothing_256)

#define DIO0_MODE_MASK                    0xC0
enum EDIO0Mode
{
  DIO0Mode_RxDone =  (0 << 6),
  DIO0Mode_TxDone =  (1 << 6),
  DIO0Mode_CadDone = (2 << 6),
};

#define CHECK_DIO0_MODE(mode) \
assert((mode == DIO0Mode_RxDone)  || \
       (mode == DIO0Mode_TxDone)  || \
       (mode == DIO0Mode_CadDone))

#define CRC_ON_PAYLOAD (1 << 6)

#endif //RFM9xWCONFIG_H

