/*********************************************************************
* Filename:   sha256.h
* Author:     Brad Conte (brad AT bradconte.com)
* Copyright:
* Disclaimer: This code is presented "as is" without any guarantees.
* Details:    Defines the API for the corresponding SHA1 implementation.
*********************************************************************/

#ifndef SHA256_H
#define SHA256_H

#include "inclstdint.h"

#ifdef __cplusplus
  extern "C" {
#endif

/*************************** HEADER FILES ***************************/
#include <stddef.h>

/****************************** MACROS ******************************/
#define SHA256_BLOCK_SIZE 32            // SHA256 outputs a 32 byte digest

/**************************** DATA TYPES ****************************/
//typedef unsigned char BYTE;             // 8-bit byte
//typedef unsigned int  WORD;             // 32-bit word, change to "long" for 16-bit machines

typedef struct {
	uint8_t data[64];
	uint32_t datalen;
	unsigned long long bitlen;
	uint32_t state[8];
} SHA256_CTX;

/*********************** FUNCTION DECLARATIONS **********************/
void WSS_sha256_init(SHA256_CTX *ctx);
void WSS_sha256_update(SHA256_CTX *ctx, const uint8_t data[], size_t len);
void WSS_sha256_final(SHA256_CTX *ctx, uint8_t hash[]);

//Anything from this point is added by Bob van Loosen, WSS Infocard systems.

#include "inclstdint.h"

typedef struct HMAC_SHA256Context
{
  SHA256_CTX ictx;
  SHA256_CTX octx;
}
HMAC_SHA256_CTX;

void HMAC_SHA256_Init(HMAC_SHA256_CTX* ctx, const uint8_t* key, size_t keylen); //do not use
void HMAC_SHA256_Init_rfc2104(HMAC_SHA256_CTX* ctx, const uint8_t* key, size_t keylen);
void HMAC_SHA256_Update(HMAC_SHA256_CTX* ctx, const uint8_t* data, size_t len);
void HMAC_SHA256_Final(HMAC_SHA256_CTX* ctx, uint8_t* hash);

#ifdef __cplusplus
  } //extern "C"
#endif

#endif   // SHA256_H
