#ifndef SPI_H
#define SPI_H

#include <stm32f4xx.h>

#include <FreeRTOS.h>
#include <semphr.h>

#include "dma.h"

class CSpi : public CDmaIrq
{
  public:
    CSpi();

    void                Setup(SPI_TypeDef* spi, uint16_t baudrateprescaler = SPI_BaudRatePrescaler_256, uint16_t spinss = SPI_NSS_Soft,
                              uint16_t pol = SPI_CPOL_Low, uint16_t pha = SPI_CPHA_1Edge);
    void                Close();

    void                Transceive(const uint8_t* output, uint8_t* input, uint32_t numbytes);

  private:
    void                SetClock(FunctionalState state);
    virtual void        DMAIRQHandler();

    uint16_t            m_spinss;
    SPI_TypeDef*        m_spi;
    DMA_Stream_TypeDef* m_txstream;
    DMA_Stream_TypeDef* m_rxstream;
    uint32_t            m_txchannel;
    uint32_t            m_rxchannel;
    uint32_t            m_lastdisable;

    SemaphoreHandle_t   m_rxdone;
};

#endif //SPI_H
