#ifndef SPIFLASH_H
#define SPIFLASH_H

#include <FreeRTOS.h>
#include <semphr.h>

#include "spi.h"
#include "gpiopin.h"
#include "inclstdint.h"

enum EMatchType
{
  MatchType_Normal,
  MatchType_Not,
};

class CSPIFlash
{
  public:
    CSPIFlash();
    ~CSPIFlash();

    bool              Setup(SPI_TypeDef* spiptr, SGPIOPin cs, SGPIOPin reset, SGPIOPin wp, SGPIOPin hold);
    void              Close();
    bool              Enabled() { return m_enabled; }

    bool              Test();

    virtual bool      Erase(uint32_t sector, uint32_t numsectors, bool* isalreadyerased = nullptr) = 0;
    virtual bool      Read(uint32_t address, uint8_t* buf, uint32_t size) = 0;
    virtual bool      Write(uint32_t address, const uint8_t* buf, uint32_t size, bool waitfinish = true) = 0;
    virtual int32_t   FindByte(uint32_t address, uint32_t size, uint8_t match, EMatchType matchtype = MatchType_Normal) = 0;
    virtual bool      IsErased(uint32_t address, uint32_t size) = 0;
    virtual bool      IsSectorErased(uint32_t sector) = 0;

    virtual uint32_t  Size()       = 0;
    virtual uint32_t  SectorSize() = 0;
    virtual uint32_t  PageSize()   = 0;

    uint32_t          NumSectors() { return Size() / SectorSize(); }

    //returns the minimum write size where the internal ECC of the SPI flash still works
    virtual uint32_t  ECCSize()    = 0;

    //cast operator to a SemaphoreHandle_t reference so that CLock(*this); can be used
    operator          SemaphoreHandle_t&() { return m_mutex; }

  protected:
    virtual bool      SetupInternal() = 0;
    virtual void      CloseInternal() = 0;

    bool              m_enabled;

    void              Select();
    void              Deselect();

    SPI_TypeDef*      m_spiptr;
    CSpi              m_spi;

    SGPIOPin          m_cs;
    SGPIOPin          m_reset;
    SGPIOPin          m_wp;
    SGPIOPin          m_hold;

    SemaphoreHandle_t m_mutex;
};

#endif //SPIFLASH_H
