#ifndef SPIFLASHLOG_H
#define SPIFLASHLOG_H

#include "spiflash.h"
#include "taskhandler.h"
#include "fwupdate.h"
#include "spiflashaddrs.h"

#include <queue.h>
#include <semphr.h>

#define EXPORT_LOG        0 //!< Set to 1 to export a stored log to the serial port.

#define LOGQUEUESIZE      (4 * 1024)
#define LOGQUEUETRIGGER   (LOGQUEUESIZE - 32)

class CSPIFlashLog : public CTaskHandler
{
  public:
    CSPIFlashLog();
    ~CSPIFlashLog();

    void              Setup(CSPIFlash* spiflash);
    CSPIFlash*        SPIFlash() { return m_spiflash; }
    void              WriteDebugLogChar(char ch);
    void              CheckErase(bool sectorfree = false);

  private:
    virtual void      Process();
    bool              ReadBackupWriteAddressAndFreeSpace();
    void              FindLogWriteAddressAndSize();

    bool              m_enabled;
    CSPIFlash*        m_spiflash;

    QueueHandle_t     m_queue;      //log characters that should be written to the SPI flash are queued here
    SemaphoreHandle_t m_mutex;      //atomic write access for the queue and the line signal
    SemaphoreHandle_t m_linesignal; //signaled when a newline is written, or the queue is almost full

    //these are protected by locking *m_spiflash
    uint32_t          m_writeaddr;
    uint32_t          m_free;
};

extern CSPIFlashLog g_spiflashlog;

#endif //SPIFLASHLOG_H
