#ifndef STM32WSS_H
#define STM32WSS_H

#include <stm32f4xx.h>
#include "inclstdint.h"

/*! Special value that signals an invalid device ID, this is the default value
    of the OTP registers. This value can also be used as a "not set" value for
    variables that hold a device ID.*/
#define DEVICE_ID_INVALID  0xFFFFFFFF

/*! Special values that can be used for something else.*/
#define DEVICE_ID_RESERVED_START 0xFFFFFF00
#define DEVICE_ID_RESERVED_END   0xFFFFFFFE

void        BoardInit();
void        SetupWatchdog();
void        SetRebootTime(int64_t rebootmillis);
bool        SoftwareRebootTimeHappened();
void        FatalError();
void        ResetFlagsStore();
uint32_t    ResetFlagMask(uint32_t reason);
uint32_t    ResetFlags();
const char* ResetFlagStr(uint32_t* resetflags);
void        PrintResetReason();
void        PrintBootInfo();
uint32_t    DeviceID();
const char* DeviceIDStr();
const char* FirmwareVersion();

#endif //STM32WSS_H
