#include "stm32wss.h"
#include "taskhandler.h"
#include "log.h"

#include <stm32f4xx.h>

CTaskHandler::CTaskHandler()
{
  m_task = 0;
}

CTaskHandler::~CTaskHandler()
{
}

void CTaskHandler::Task(void *pvParameters)
{
  PrintLog(LOGTASK, "Task %s started\n", pcTaskGetName(((CTaskHandler*)pvParameters)->m_task));

  //call the virtual Process function, this should be declared in a class derived from this class
  ((CTaskHandler*)pvParameters)->Process();

  PrintLog(LOGTASK, "Task %s ended\n", pcTaskGetName(((CTaskHandler*)pvParameters)->m_task));

  //delete the task when the Process function ends
  vTaskDelete(((CTaskHandler*)pvParameters)->m_task);
}

void CTaskHandler::Create(const char* name, uint32_t priority /*= DEFAULT_PRIORITY*/, uint32_t stackdepth /*= DEFAULT_STACKDEPTH*/)
{
  BaseType_t retval = xTaskCreate(CTaskHandler::Task, name, stackdepth, this, priority, &m_task);
  if (retval != pdPASS)
  { //consider failing to start a task fatal, because it can only mean that FreeRTOS is out of memory
    DEBUGUSARTprintf("ERROR: Task %s failed to start, error %li\n", name, retval);
    FatalError();
  }
}

void CTaskHandler::Suspend()
{
  if (m_task)
    vTaskSuspend(m_task);
}

void CTaskHandler::Resume()
{
  if (m_task)
    vTaskResume(m_task);
}

void CTaskHandler::Notify()
{
  if (m_task)
    xTaskNotifyGive(m_task);
}

void CTaskHandler::NotifyFromISR()
{
  if (m_task)
  {
    //if the FreeRTOS task is blocked on waiting for a notification
    //use portYIELD_FROM_ISR to unblock it immediately when this ISR is finished
    BaseType_t taskwoken = pdFALSE;
    vTaskNotifyGiveFromISR(m_task, &taskwoken);
    portYIELD_FROM_ISR(taskwoken);
  }
}

void CTaskHandler::WaitNotificationCounting()
{
  //wait for a notification, and decrease the notification counter on exit
  ulTaskNotifyTake(pdFALSE, portMAX_DELAY);
}

void CTaskHandler::WaitNotificationBinary()
{
  //wait for a notification, and clear the notification counter on exit
  ulTaskNotifyTake(pdTRUE, portMAX_DELAY);
}

void CTaskHandler::ClearNotification()
{
  //clear the notification with a delay of zero
  ulTaskNotifyTake(pdTRUE, 0);
}
